@@ -342,6 +342,21 @@ private:
         details::grid::sync(&_data.grid.gridWs->barrier);
     }
 
+#if defined(_CG_CPP11_FEATURES)
+    using arrival_token = unsigned int;
+
+    _CG_QUALIFIER arrival_token barrier_arrive() const {
+        if (!is_valid()) {
+            _CG_ABORT();
+        }
+        return details::grid::barrier_arrive(&_data.grid.gridWs->barrier);
+    }
+
+    _CG_QUALIFIER void barrier_wait(arrival_token&& token) const {
+        details::grid::barrier_wait(token, &_data.grid.gridWs->barrier);
+    }
+#endif
+
     _CG_STATIC_QUALIFIER unsigned long long size() {
         return details::grid::size();
     }
@@ -610,6 +625,18 @@ class thread_block : public thread_group_base<details::thread_block_id>
         details::cta::sync();
     }
 
+#if defined(_CG_CPP11_FEATURES)
+    struct arrival_token {};
+
+    _CG_QUALIFIER arrival_token barrier_arrive() const {
+        return arrival_token();
+    }
+
+    _CG_QUALIFIER void barrier_wait(arrival_token&&) const {
+        details::cta::sync();
+    }
+#endif
+
     _CG_STATIC_QUALIFIER unsigned int size() {
         return details::cta::size();
     }
@@ -1681,6 +1708,9 @@ _CG_QUALIFIER unsigned long long thread_group::thread_rank() const
 _CG_END_NAMESPACE
 
 #include <cooperative_groups/details/partitioning.h>
+#if (!defined(_MSC_VER) || defined(_WIN64))
+# include <cooperative_groups/details/invoke.h>
+#endif
 
 # endif /* ! (__cplusplus, __CUDACC__) */
 
