@@ -127,18 +127,12 @@ namespace details {
 
     class _coalesced_group_data_access {
     public:
-        // Retrieve mask of coalesced groups
+        // Retrieve mask of coalesced groups and tiles
         template <typename TyGroup>
         _CG_STATIC_QUALIFIER unsigned int get_mask(const TyGroup &group) {
             return group.get_mask();
         }
 
-        // Retrieve mask of tiles
-        template <template <typename, typename> typename TyGroup, typename Sz, typename Parent>
-        _CG_STATIC_QUALIFIER unsigned int get_mask(const TyGroup<Sz, Parent> &group) {
-            return group.build_maks();
-        }
-
         template <typename TyGroup>
         _CG_STATIC_QUALIFIER TyGroup construct_from_mask(unsigned int mask) {
             return TyGroup(mask);
@@ -375,10 +369,16 @@ namespace details {
     };
 
     namespace grid {
-        _CG_STATIC_QUALIFIER void sync(unsigned int *bar) {
-            unsigned int expected = gridDim.x * gridDim.y * gridDim.z;
+        _CG_STATIC_QUALIFIER unsigned int barrier_arrive(unsigned int *bar) {
+            return details::sync_grids_arrive(bar);
+        }
 
-            details::sync_grids(expected, bar);
+        _CG_STATIC_QUALIFIER void barrier_wait(unsigned int token, unsigned int *bar) {
+            details::sync_grids_wait(token, bar);
+        }
+
+        _CG_STATIC_QUALIFIER void sync(unsigned int *bar) {
+            details::sync_grids(bar);
         }
 
         _CG_STATIC_QUALIFIER unsigned long long num_blocks()
