@@ -293,7 +293,7 @@ namespace details {
     template <>
     struct tile_async_reduce_dispatch<details::coalesced_group_id> {
         template <unsigned int TySize, typename ParentT, typename TyDst, typename TyVal, typename TyFn, typename TyResHandler>
-        _CG_STATIC_QUALIFIER void reduce(const __single_warp_thread_block_tile<TySize, ParentT>& group, TyDst& dst, TyVal&& val, TyFn&& op, TyResHandler& res_handler) {
+        _CG_STATIC_QUALIFIER void reduce(const thread_block_tile<TySize, ParentT>& group, TyDst& dst, TyVal&& val, TyFn&& op, TyResHandler& res_handler) {
             // Do regular, in group reduction
             auto result = details::reduce(group, _CG_STL_NAMESPACE::forward<TyVal>(val), op);
 
