@@ -1,5 +1,5 @@
 /*
- * Copyright 1993-2018 NVIDIA Corporation.  All rights reserved.
+ * Copyright 1993-2023 NVIDIA Corporation.  All rights reserved.
  *
  * NOTICE TO LICENSEE:
  *
@@ -55,7 +55,7 @@
 #define __UNDEF_CUDA_INCLUDE_COMPILER_INTERNAL_HEADERS_CUDA_RUNTIME_H__
 #endif
 
-#if !defined(__CUDACC_RTC__)
+#define EXCLUDE_FROM_RTC
 #if defined(__GNUC__)
 #if defined(__clang__) || (!defined(__PGIC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6)))
 #pragma GCC diagnostic push
@@ -67,13 +67,12 @@
 #pragma warning(push)
 #pragma warning(disable: 4820)
 #endif
-#endif
-
 #ifdef __QNX__
 #if (__GNUC__ == 4 && __GNUC_MINOR__ >= 7)
 typedef unsigned size_t;
 #endif
 #endif
+#undef EXCLUDE_FROM_RTC
 /*******************************************************************************
 *                                                                              *
 *                                                                              *
@@ -142,6 +141,7 @@ struct  __device_builtin__ __nv_lambda_preheader_injection { };
 #endif
 /** \endcond impl_private */
 
+#define EXCLUDE_FROM_RTC
 #if defined(__cplusplus) && !defined(__CUDACC_RTC__)
 
 #if __cplusplus >= 201103
@@ -286,7 +286,7 @@ static __inline__ __host__ cudaError_t cudaLaunchKernelEx(
         return ::cudaLaunchKernelExC(config, (const void *)kernel, pArgs);
     }(std::forward<ActTypes>(args)...);
 }
-# endif
+#endif
 
 /**
  *\brief Launches a device function
@@ -635,6 +635,37 @@ static __inline__ __host__ cudaError_t cudaMallocManaged(
   return ::cudaMallocManaged((void**)(void*)devPtr, size, flags);
 }
 
+/**
+ * \brief Advise about the usage of a given memory range.
+ *
+ * This is an alternate spelling for cudaMemAdvise made available through operator overloading.
+ *
+ * \sa ::cudaMemAdvise,
+ * \ref ::cudaMemAdvise(const void* devPtr, size_t count, enum cudaMemoryAdvise advice, struct cudaMemLocation location)  "cudaMemAdvise (C API)"
+ */
+template<class T>
+cudaError_t cudaMemAdvise(
+  T                      *devPtr,
+  size_t                 count,
+  enum cudaMemoryAdvise  advice,
+  struct cudaMemLocation location
+)
+{
+  return ::cudaMemAdvise_v2((const void *)devPtr, count, advice, location);
+}
+
+template<class T>
+static __inline__ __host__ cudaError_t cudaMemPrefetchAsync(
+  T                       *devPtr,
+  size_t                  count,
+  struct cudaMemLocation  location,
+  unsigned int            flags,
+  cudaStream_t            stream = 0
+)
+{
+  return ::cudaMemPrefetchAsync_v2((const void *)devPtr, count, location, flags, stream);
+}
+
 /**
  * \brief Attach memory to a stream asynchronously
  *
@@ -2251,6 +2282,29 @@ static __inline__ __host__ cudaError_t cudaFuncSetAttribute(
   return ::cudaFuncSetAttribute((const void*)entry, attr, value);
 }
 
+/**
+ * \brief Get pointer to device kernel that matches entry function \p entryFuncAddr
+  *
+  * Returns in \p kernelPtr the device kernel corresponding to the entry function \p entryFuncAddr.
+  *
+  * \param kernelPtr          - Returns the device kernel
+  * \param entryFuncAddr      - Address of device entry function to search kernel for
+  *
+  * \return
+  * ::cudaSuccess
+  *
+  * \sa
+  * \ref ::cudaGetKernel(cudaKernel_t *kernelPtr, const void *entryFuncAddr) "cudaGetKernel (C API)"
+  */
+template<class T>
+static  __inline__ __host__ cudaError_t cudaGetKernel(
+  cudaKernel_t *kernelPtr,
+  const T *entryFuncAddr
+)
+{
+  return ::cudaGetKernel(kernelPtr, (const void *)entryFuncAddr);
+}
+
 #endif /* __CUDACC__ */
 
 /** @} */ /* END CUDART_HIGHLEVEL */
@@ -2267,6 +2321,7 @@ static __inline__ __host__ cudaError_t cudaFuncSetAttribute(
 #endif
 #endif
 
+#undef EXCLUDE_FROM_RTC
 #undef __CUDA_DEPRECATED
 
 #if defined(__UNDEF_CUDA_INCLUDE_COMPILER_INTERNAL_HEADERS_CUDA_RUNTIME_H__)
