@@ -2,7 +2,7 @@
 #define NVPERF_HOST_H
 
 /*
- * Copyright 2014-2022  NVIDIA Corporation.  All rights reserved.
+ * Copyright 2014-2023 NVIDIA Corporation.  All rights reserved.
  *
  * NOTICE TO USER:
  *
@@ -1053,12 +1053,14 @@ extern "C" {
         NVPW_DIM_UNIT_ATTRIBUTES = 3776338729,
         NVPW_DIM_UNIT_BYTES = 3797850191,
         NVPW_DIM_UNIT_CTAS = 1960564139,
+        NVPW_DIM_UNIT_CTC_CYCLES = 2224883873,
         NVPW_DIM_UNIT_DRAM_CYCLES = 2650981327,
         NVPW_DIM_UNIT_FBP_CYCLES = 1785238957,
         NVPW_DIM_UNIT_FE_OPS = 2919159083,
         NVPW_DIM_UNIT_GPC_CYCLES = 1222631184,
         NVPW_DIM_UNIT_IDC_REQUESTS = 2012649669,
         NVPW_DIM_UNIT_INSTRUCTIONS = 1418625543,
+        NVPW_DIM_UNIT_KILOBYTES = 1335980302,
         NVPW_DIM_UNIT_L1DATA_BANK_ACCESSES = 1479493682,
         NVPW_DIM_UNIT_L1DATA_BANK_CONFLICTS = 3433170787,
         NVPW_DIM_UNIT_L1TEX_REQUESTS = 1306473767,
@@ -1067,6 +1069,7 @@ extern "C" {
         NVPW_DIM_UNIT_L2_REQUESTS = 1143695106,
         NVPW_DIM_UNIT_L2_SECTORS = 3424101564,
         NVPW_DIM_UNIT_L2_TAGS = 3755612781,
+        NVPW_DIM_UNIT_MCC_CYCLES = 1826685787,
         NVPW_DIM_UNIT_NANOSECONDS = 3047500672,
         NVPW_DIM_UNIT_NVLRX_CYCLES = 4059934930,
         NVPW_DIM_UNIT_NVLTX_CYCLES = 1814350488,
@@ -1105,6 +1108,7 @@ extern "C" {
         NVPW_HW_UNIT_IDC = 842765289,
         NVPW_HW_UNIT_L1TEX = 893940957,
         NVPW_HW_UNIT_LTS = 2333266697,
+        NVPW_HW_UNIT_MCC = 3980130194,
         NVPW_HW_UNIT_NVLRX = 3091684901,
         NVPW_HW_UNIT_NVLTX = 869679659,
         NVPW_HW_UNIT_PCIE = 3433264174,
