@@ -83,8 +83,8 @@ extern "C" {
 /*--------------------------------- API Version ------------------------------*/
 
 #define CUDBG_API_VERSION_MAJOR      12 /* Major release version number */
-#define CUDBG_API_VERSION_MINOR       0 /* Minor release version number */
-#define CUDBG_API_VERSION_REVISION  133 /* Revision (build) number */
+#define CUDBG_API_VERSION_MINOR       2 /* Minor release version number */
+#define CUDBG_API_VERSION_REVISION  134 /* Revision (build) number */
 
 /*---------------------------------- Constants -------------------------------*/
 
@@ -1078,6 +1078,9 @@ struct CUDBGAPI_st {
     CUDBGResult (*getClusterDim)(uint32_t dev, uint64_t gridId64, CuDim3 *clusterDim);
     CUDBGResult (*readWarpState)(uint32_t dev, uint32_t sm, uint32_t wp, CUDBGWarpState *state);
     CUDBGResult (*readClusterIdx)(uint32_t dev, uint32_t sm, uint32_t wp, CuDim3 *clusterIdx);
+
+    /* 12.2 Extensions */
+    CUDBGResult (*getErrorStringEx)(char *buf, uint32_t bufSz, uint32_t *msgSz);
 };
 
 #ifdef __cplusplus
