@@ -1,11 +1,11 @@
 var indexerLanguage="en";
 //Auto generated index for searching.
-w["0,0,0"]="0*710";
+w["0,0,0"]="0*790";
 w["0-15"]="0*60";
 w["0-16"]="0*30";
 w["0-base"]="0*20";
 w["00"]="1*10";
-w["000"]="0*30";
+w["000"]="0*70";
 w["00000"]="0*150";
 w["00000000"]="0*30";
 w["00001"]="0*90";
@@ -15,14 +15,16 @@ w["024"]="0*90";
 w["037"]="0*60";
 w["0400"]="0*20";
 w["07"]="0*150";
-w["0x0"]="0*60";
-w["0x1"]="14*100";
+w["0x0"]="0*60,14*60";
+w["0x1"]="14*120";
 w["0x10"]="14*20";
+w["0x1000400"]="0*40";
 w["0x1030"]="0*30";
 w["0x1080b"]="0*30";
 w["0x110"]="0*30";
 w["0x14"]="0*20";
 w["0x160"]="0*90";
+w["0x170"]="0*40";
 w["0x19b0"]="0*20";
 w["0x2"]="14*60";
 w["0x20"]="14*20";
@@ -61,9 +63,8 @@ w["0x7fb649a00000"]="0*30";
 w["0x7fb654c00000"]="0*30";
 w["0x7fed9f400000"]="0*20";
 w["0x7fed9f400014"]="0*20";
-w["0x7ff"]="14*20";
-w["0x7fffffff"]="14*460";
-w["0x8"]="14*20";
+w["0x7fffffff"]="14*520";
+w["0x8"]="14*40";
 w["0x80"]="14*20";
 w["0x87654320"]="0*30";
 w["0x8c"]="0*30";
@@ -83,46 +84,47 @@ w["0xb3ac"]="0*60";
 w["0xb535"]="0*120";
 w["0xf0"]="0*80";
 w["0xfccb"]="0*120";
+w["0xffffffff"]="14*60";
 w["1,0,0"]="0*20";
 w["1,024"]="0*90";
-w["1,49"]="35*10";
+w["1,49"]="36*10";
 w["1,thread"]="0*60";
 w["1-base"]="0*20";
-w["1.1"]="0*38,1*20,13*10,14*38,34*38,35*38";
+w["1.1"]="0*38,1*20,13*10,14*38,35*38,36*38";
 w["1.2"]="0*38,1*10,13*10,14*38";
 w["1.3"]="0*38,1*10,13*10,14*38";
 w["1.4"]="0*38,1*10,13*10,14*38";
 w["1.5"]="1*10,13*10,14*38";
 w["1.6"]="13*10,14*38";
-w["10"]="0*703,1*20,2*40,3*38,13*10,14*100,34*20";
-w["10.1"]="0*38,2*20,34*20";
-w["10.1.1"]="0*48";
-w["10.1.2"]="0*48";
-w["10.1.3"]="0*48";
-w["10.1.4"]="0*48";
-w["10.2"]="0*38";
-w["10.2.1"]="0*48";
-w["10.2.2"]="0*48";
-w["10.3"]="0*38";
-w["10.3.1"]="0*48";
-w["10.4"]="0*38";
-w["10.4.1"]="0*48";
-w["10.4.2"]="0*48";
-w["100"]="0*40,2*20";
-w["10000"]="0*20";
+w["10"]="0*255,1*20,2*40,3*38,13*10,14*100,35*20";
+w["10,000,000"]="0*20";
+w["10.1"]="0*38,2*20,35*20";
+w["100"]="0*80,2*20";
+w["1000400"]="0*40";
 w["1024"]="0*30";
 w["1030"]="0*30";
 w["1080"]="0*30";
-w["11"]="0*60,2*20,3*38,13*10,14*100,34*20,35*40";
-w["11.0"]="2*20,34*20,35*20";
-w["11.2"]="14*20";
-w["11.3"]="35*20";
+w["11"]="0*673,2*20,3*38,13*10,14*100,35*20,36*40";
+w["11.0"]="2*20,35*20,36*20";
+w["11.1"]="0*38";
+w["11.1.1"]="0*48";
+w["11.1.2"]="0*48";
+w["11.1.3"]="0*48";
+w["11.1.4"]="0*48";
+w["11.2"]="0*38,14*20";
+w["11.2.1"]="0*48";
+w["11.2.2"]="0*48";
+w["11.3"]="0*38,36*20";
+w["11.3.1"]="0*48";
+w["11.4"]="0*38";
+w["11.4.1"]="0*48";
+w["11.4.2"]="0*48";
 w["11.6"]="0*40";
 w["11.7"]="0*20";
 w["110"]="0*60";
 w["112"]="0*30";
-w["12"]="0*30,1*10,3*38,13*10,14*60,35*20";
-w["12.0"]="35*20";
+w["12"]="0*30,1*10,3*38,13*10,14*80,36*20";
+w["12.0"]="36*20";
 w["128"]="0*90";
 w["13"]="0*30,3*38,13*10,14*60";
 w["132"]="0*20";
@@ -131,13 +133,14 @@ w["140"]="0*60";
 w["140,276,689,190,112"]="0*30";
 w["140,418,624,437,472"]="0*30";
 w["15"]="0*90,3*38,13*10,14*60";
-w["16"]="0*310,3*38,13*10,14*100,35*120";
+w["16"]="0*310,3*38,13*10,14*100,36*120";
 w["160"]="0*90";
 w["17"]="0*30,3*38,13*10,14*40";
-w["18"]="0*0,1*0,2*0,3*38,5*0,13*10,14*40,15*0,34*0,35*0";
-w["19"]="0*80,1*10,14*40";
+w["170"]="0*40";
+w["18"]="3*38,13*10,14*40";
+w["19"]="0*80,1*10,3*38,13*10,14*40";
 w["190"]="0*30";
-w["2.1"]="0*38,1*10,3*38,13*10,34*86,35*38";
+w["2.1"]="0*38,1*10,3*38,13*10,35*38,36*38";
 w["2.10"]="3*38,13*10";
 w["2.11"]="3*38,13*10";
 w["2.12"]="3*38,13*10";
@@ -147,20 +150,22 @@ w["2.15"]="3*38,13*10";
 w["2.16"]="3*38,13*10";
 w["2.17"]="3*38,13*10";
 w["2.18"]="3*38,13*10";
-w["2.2"]="0*38,1*10,3*38,13*10,34*38,35*38";
-w["2.2.2"]="34*48";
-w["2.2.3"]="34*48";
-w["2.2.4"]="34*48";
-w["2.2.5"]="34*48";
-w["2.2.6"]="34*48";
-w["2.2.7"]="34*48";
-w["2.3"]="0*38,1*10,3*38,13*10,34*38,35*38";
-w["2.3.1"]="34*48,35*48";
-w["2.3.2"]="34*48,35*48";
-w["2.4"]="3*38,13*10,34*38,35*38";
-w["2.5"]="3*38,13*10,35*38";
-w["2.5.1"]="35*48";
-w["2.5.2"]="35*48";
+w["2.19"]="3*38,13*10";
+w["2.2"]="0*38,1*10,3*38,13*10,35*38,36*38";
+w["2.2.1"]="35*48";
+w["2.2.2"]="35*48";
+w["2.2.3"]="35*48";
+w["2.2.4"]="35*48";
+w["2.2.5"]="35*48";
+w["2.2.6"]="35*48";
+w["2.2.7"]="35*48";
+w["2.3"]="0*38,1*10,3*38,13*10,35*38,36*38";
+w["2.3.1"]="35*48,36*48";
+w["2.3.2"]="35*48,36*48";
+w["2.4"]="3*38,13*10,35*38,36*38";
+w["2.5"]="3*38,13*10,35*38,36*38";
+w["2.5.1"]="35*48,36*48";
+w["2.5.2"]="36*48";
 w["2.6"]="3*38,13*10";
 w["2.7"]="3*38,13*10";
 w["2.8"]="3*38,13*10";
@@ -169,8 +174,8 @@ w["20"]="0*150,14*60";
 w["2012"]="1*20";
 w["2012-2016"]="1*20";
 w["2014"]="1*20";
-w["2016"]="1*20,2*20";
-w["2019"]="0*20,1*20,2*58,15*20,34*20";
+w["2016"]="1*20";
+w["2019"]="0*20,1*20,2*58,15*20,35*20";
 w["2019.1"]="2*38";
 w["2020"]="2*266";
 w["2020.1"]="2*38";
@@ -180,15 +185,16 @@ w["2020.2"]="2*38";
 w["2020.2.1"]="2*38";
 w["2020.3"]="2*38";
 w["2020.3.1"]="2*38";
-w["2021"]="2*266,35*20";
+w["2021"]="2*304,36*20";
 w["2021.1"]="2*38";
 w["2021.1.1"]="2*38";
 w["2021.2"]="2*38";
 w["2021.2.1"]="2*38";
 w["2021.2.2"]="2*38";
+w["2021.2.3"]="2*38";
 w["2021.3"]="2*38";
 w["2021.3.1"]="2*38";
-w["2022"]="0*0,1*0,2*266,3*0,5*0,13*0,14*0,15*0,34*0,35*0";
+w["2022"]="2*266";
 w["2022.1"]="2*38";
 w["2022.1.1"]="2*38";
 w["2022.2"]="2*38";
@@ -196,14 +202,20 @@ w["2022.2.1"]="2*38";
 w["2022.3"]="2*38";
 w["2022.4"]="2*38";
 w["2022.4.1"]="2*38";
-w["2023"]="0*20,1*20,2*20,3*0,5*0,13*0,14*0,15*20,34*20,35*20";
-w["21"]="0*20,14*20";
-w["22"]="0*90,14*20";
+w["2023"]="0*21,1*21,2*135,3*1,5*1,13*1,14*1,15*21,35*21,36*21";
+w["2023.1"]="2*38";
+w["2023.1.1"]="2*38";
+w["2023.2"]="2*38";
+w["21"]="0*20,14*40";
+w["22"]="0*90,1*0,2*0,3*0,5*0,13*0,14*40,15*0,35*0,36*0";
 w["227"]="1*20";
-w["23"]="14*20";
-w["24"]="14*20";
+w["23"]="14*40";
+w["24"]="14*40";
+w["25"]="14*20";
+w["26"]="14*20";
 w["262"]="0*20";
 w["262,132"]="0*20";
+w["27"]="14*20";
 w["2701"]="1*20";
 w["276"]="0*30";
 w["2774"]="0*120";
@@ -221,8 +233,8 @@ w["3.8"]="0*38";
 w["3.9"]="0*38";
 w["3/4"]="0*30";
 w["30"]="0*60,1*10";
-w["3020"]="34*90";
-w["32"]="0*310,2*20,3*760,14*1500,34*60,35*60";
+w["3020"]="35*90";
+w["32"]="0*310,2*20,3*800,14*2140,35*60,36*60";
 w["32,0,0"]="0*60";
 w["32-bit"]="0*20,2*20";
 w["32bit"]="0*60";
@@ -234,6 +246,7 @@ w["37,0,0"]="0*20";
 w["379"]="0*150";
 w["39"]="0*30";
 w["4.1"]="0*38,2*38";
+w["4.10"]="0*38";
 w["4.2"]="0*38,2*38";
 w["4.3"]="0*38";
 w["4.4"]="0*38";
@@ -257,7 +270,7 @@ w["46"]="0*20";
 w["460"]="0*60";
 w["472"]="0*30";
 w["48"]="0*30";
-w["49"]="35*20";
+w["49"]="36*20";
 w["49152"]="0*20";
 w["5.0"]="2*20";
 w["5.1"]="0*38";
@@ -271,7 +284,7 @@ w["515"]="14*60";
 w["52"]="1*10";
 w["52.227-19"]="1*10";
 w["520"]="0*25";
-w["535"]="0*120";
+w["535"]="0*120,2*20,3*40,14*80";
 w["55"]="0*80";
 w["569918"]="0*20";
 w["578"]="0*150";
@@ -280,13 +293,15 @@ w["6.1"]="0*38";
 w["6.2"]="0*38";
 w["6.3"]="0*38";
 w["6.4"]="0*38";
+w["6.5"]="0*38";
 w["60"]="0*20,1*10";
 w["612"]="0*30";
 w["618"]="0*30";
 w["624"]="0*30";
-w["64"]="0*660,2*140,3*640,14*1560,34*30,35*90";
+w["64"]="0*660,2*140,3*720,14*1880,35*30,36*90";
 w["64,0,0"]="0*30";
 w["649"]="0*30";
+w["64_t"]="3*20";
 w["64bit"]="0*20";
 w["6510"]="0*60";
 w["654"]="0*30";
@@ -299,8 +314,8 @@ w["7.2"]="0*38";
 w["7.3"]="0*38";
 w["7.4"]="0*38";
 w["7.5"]="0*38";
-w["7.6"]="0*58,2*20";
-w["7.7"]="0*38";
+w["7.6"]="0*38,2*20";
+w["7.7"]="0*38,2*20";
 w["7.8"]="0*38";
 w["7.9"]="0*38";
 w["70"]="0*80";
@@ -308,20 +323,22 @@ w["7202"]="1*10";
 w["77"]="0*20";
 w["7822"]="0*20";
 w["79"]="0*60";
-w["8.0"]="0*20";
+w["8.0"]="0*40,2*20";
 w["8.1"]="0*38,1*10";
-w["8.2"]="0*38,1*10";
+w["8.2"]="1*10";
 w["8.9"]="2*20";
 w["80"]="0*70,14*20";
 w["803"]="0*120";
 w["81"]="0*20";
 w["85"]="0*20";
 w["86"]="0*500,2*40";
+w["86_64-linux-gnu/libc.so.6"]="0*30";
 w["87654320"]="0*30";
 w["9.0"]="0*30";
 w["9.1"]="0*38";
-w["9.x"]="2*20";
-w["90"]="0*30";
+w["9.2"]="0*38";
+w["9.x"]="2*40";
+w["90"]="0*70";
 w["91"]="0*40";
 w["920"]="0*40";
 w["94"]="0*30";
@@ -330,91 +347,93 @@ w["953"]="0*30";
 w["9544"]="0*30";
 w["9840"]="0*20";
 w["999"]="14*20";
-w["a::vis"]="14*20";
+w["a/github/compute-sanitizer-samples/memcheck/memcheck_demo"]="0*30";
 w["aarch"]="2*60";
 w["aarch64"]="2*40";
 w["aarch64sbsa"]="2*20";
 w["abandon"]="1*10";
 w["abl"]="0*70,2*10";
 w["abort"]="0*20,2*20";
-w["about"]="0*448,1*10,2*10,14*180,35*20";
-w["abov"]="0*170,1*100,2*20,14*40,34*30";
+w["about"]="0*468,1*10,14*200,36*20";
+w["abov"]="0*170,1*100,2*20,14*40,35*30";
 w["absenc"]="1*10";
 w["absent"]="1*10";
 w["ac"]="0*90";
 w["academ"]="1*43";
-w["acc"]="14*40";
 w["acceler"]="2*30";
 w["accept"]="0*40,1*100";
-w["access"]="0*2210,1*110,2*130,3*160,14*1000,34*300,35*110";
-w["accesss"]="14*180,34*30";
+w["access"]="0*2330,1*110,2*170,3*160,14*1180,35*300,36*110";
+w["accesss"]="14*300,35*30";
 w["accord"]="1*30";
-w["accur"]="0*80,1*20,2*40,15*20,34*20,35*20";
+w["account"]="14*20,35*30";
+w["accur"]="0*80,1*20,2*40,15*20,35*20,36*20";
 w["accuraci"]="1*20";
-w["achiev"]="0*40,35*20";
+w["achiev"]="0*60,36*20";
 w["acknowledg"]="1*100";
 w["acquir"]="1*10";
-w["across"]="0*20";
+w["across"]="0*40,14*60";
 w["act"]="1*20";
 w["action"]="0*278,1*30,14*180";
-w["activ"]="0*60,1*30,14*20,35*30";
-w["actual"]="0*40,14*60,35*20";
-w["ad"]="0*60,2*1080";
+w["activ"]="0*80,1*30,14*20,36*30";
+w["actual"]="0*40,14*60,36*20";
+w["ad"]="0*60,2*1300";
 w["ad10x/sm"]="2*20";
-w["ada"]="0*20";
-w["add"]="0*40";
-w["addit"]="0*180,1*60,2*10,35*20";
-w["address"]="0*580,2*20,3*280,5*10,14*560,35*380";
+w["add"]="0*40,2*20";
+w["addit"]="0*200,1*60,2*10,36*20";
+w["address"]="0*620,2*30,3*340,5*10,14*860,36*380";
+w["addressmatrixa"]="14*60";
+w["addressmatrixb"]="14*60";
 w["adequ"]="1*10";
 w["adjust"]="0*20";
 w["administ"]="1*10";
 w["administr"]="1*20";
 w["adopt"]="1*10";
-w["advanc"]="0*20,35*48";
+w["advanc"]="0*20,36*48";
 w["advis"]="0*20,1*50";
 w["advisor"]="1*10";
 w["aff"]="0*40";
 w["affect"]="0*80,1*40";
-w["affili"]="0*20,1*93,2*20,15*20,34*20,35*20";
-w["after"]="0*590,1*10,2*70,14*80,34*20,35*30";
+w["affili"]="0*20,1*93,2*20,15*20,35*20,36*20";
+w["after"]="0*610,1*10,2*70,14*120,35*20,36*30";
 w["afterward"]="0*30";
-w["ag"]="1*10";
 w["against"]="1*20";
 w["agenc"]="1*10";
 w["agent"]="1*30";
 w["agre"]="1*150";
+w["agreem"]="1*10";
 w["agreement"]="1*705";
 w["algorithm"]="1*10";
-w["align"]="0*20,2*20,14*40";
-w["all"]="0*1060,1*400,2*120,3*20,5*10,14*590,15*80,34*210,35*270";
+w["alid"]="14*20";
+w["alid_paramet"]="14*20";
+w["align"]="0*20,2*20,14*140";
+w["all"]="0*1080,1*400,2*130,3*20,5*10,14*630,15*80,35*210,36*270";
 w["all,use-before-alloc,use-after-free,no"]="0*20";
-w["allbackcachecontrol"]="14*20";
-w["alloc"]="0*1478,2*140,3*560,14*1260,34*110,35*890";
+w["alloc"]="0*1518,2*140,3*560,14*1260,35*110,36*890";
 w["allocatedptr"]="14*60";
 w["allocateds"]="14*60";
-w["allow"]="0*130,1*30,14*40,34*70,35*160";
+w["allow"]="0*130,1*30,14*40,35*70,36*160";
 w["alon"]="1*10";
 w["along"]="0*30";
 w["alpha"]="1*20";
-w["alreadi"]="0*40,14*20";
-w["also"]="0*400,2*10,14*20,34*70";
+w["alreadi"]="0*60,14*20";
+w["also"]="0*440,2*10,14*20,35*70";
 w["alter"]="1*10";
-w["alway"]="0*60";
+w["alway"]="0*60,14*20";
+w["am"]="3*20";
+w["am_destroy_finish"]="3*20";
 w["amend"]="1*10";
 w["america"]="1*10";
 w["amount"]="1*30";
 w["amper"]="0*40,2*20";
-w["ampl"]="0*30";
-w["amples/memcheck/memcheck_demo"]="0*30";
 w["analys"]="0*20";
 w["analysi"]="0*288";
 w["analyz"]="2*10";
 w["and/or"]="1*210";
-w["ani"]="0*330,1*1150,2*40,14*140,15*40,34*130,35*120";
-w["anoth"]="0*60,34*20";
+w["ani"]="0*390,1*1150,2*40,14*140,15*40,35*160,36*120";
+w["anoth"]="0*120,14*20,35*20";
 w["anymor"]="0*20";
-w["api"]="0*863,2*340,3*1045,5*31,13*170,14*1591,15*31,34*1702,35*444";
-w["api/c/includ"]="35*30";
+w["ap"]="36*30";
+w["api"]="0*883,2*380,3*1045,5*31,13*170,14*1691,15*31,35*1812,36*414";
 w["apicallbacksit"]="3*40,14*40";
 w["apicontext"]="3*40,5*10,14*40";
 w["apistream"]="3*40,5*10,14*40";
@@ -422,38 +441,38 @@ w["app"]="0*280";
 w["app_nam"]="0*140";
 w["app_opt"]="0*140";
 w["appear"]="1*30";
-w["appl"]="0*20";
 w["applciat"]="0*20";
-w["appli"]="0*50,1*20,14*40,35*190";
-w["applic"]="0*2320,1*250,2*160,34*130,35*20";
+w["appli"]="0*50,1*20,14*40,36*190";
+w["applic"]="0*2360,1*250,2*260,14*60,35*160,36*20";
 w["application-on"]="0*20";
 w["application-only,al"]="0*20";
-w["appropri"]="0*20,1*20";
-w["approv"]="0*20,1*30,2*20,15*20,34*20,35*20";
-w["architectur"]="0*160,2*40,14*60";
+w["appr"]="1*10";
+w["appropri"]="0*80,1*10";
+w["approv"]="0*20,1*30,2*20,15*20,35*20,36*20";
+w["architectur"]="0*180,2*60,14*60";
 w["area"]="0*20";
 w["arg"]="0*20";
 w["argc"]="0*150";
-w["argument"]="0*90,2*20,3*380,14*100,34*250";
+w["argument"]="0*90,2*20,3*400,14*100,35*250";
 w["argv"]="0*150";
 w["aris"]="1*120";
 w["aros"]="1*10";
 w["array"]="3*170,14*180";
-w["arriv"]="14*240";
+w["arriv"]="14*340";
 w["arrive-on"]="14*80";
-w["ascii"]="35*70";
+w["ascii"]="36*70";
 w["aspx"]="0*20";
 w["asset"]="1*10";
-w["assign"]="0*30,1*50,35*110";
+w["assign"]="0*30,1*50,36*110";
 w["assist"]="1*10";
-w["associ"]="0*60,1*50,2*20,3*40,14*120,15*20,34*360,35*40";
-w["assum"]="0*20,1*20,2*20,15*20,34*20,35*50";
+w["associ"]="0*60,1*50,2*20,3*40,14*140,15*20,35*360,36*40";
+w["assum"]="0*20,1*20,2*20,15*20,35*20,36*50";
 w["assumpt"]="0*90";
-w["async"]="0*120,2*40,3*60,14*140";
+w["async"]="0*140,2*80,3*60,14*500";
 w["async-group"]="0*20";
-w["asynchron"]="0*118,2*40,3*40,14*180";
-w["ate"]="2*38";
-w["atom"]="0*40,2*10,14*180,35*150";
+w["asynchron"]="0*118,2*40,3*40,14*300";
+w["at"]="2*10";
+w["atom"]="0*40,2*10,14*180,36*150";
 w["atomsi"]="14*20";
 w["attach"]="0*190,3*80,14*120";
 w["attempt"]="0*100,1*10,14*40";
@@ -464,157 +483,163 @@ w["attribut"]="0*100,1*20";
 w["audio"]="1*33";
 w["audio/video"]="1*13";
 w["authent"]="1*10";
-w["author"]="0*20,1*213,2*20,15*20,34*20,35*20";
-w["auto"]="35*170";
+w["author"]="0*20,1*213,2*20,15*20,35*20,36*20";
+w["auto"]="36*170";
 w["autom"]="0*20";
-w["automat"]="0*80,1*10,2*40,35*30";
-w["avail"]="0*180,1*110,3*60,14*120";
+w["automat"]="0*80,1*10,2*30,36*30";
+w["avail"]="0*180,1*110,3*60,14*180";
 w["avion"]="1*10";
-w["avoid"]="0*120,1*10,34*20,35*20";
+w["avoid"]="0*120,1*10,35*20,36*20";
 w["await"]="0*20";
-w["awar"]="0*20,35*20";
-w["back"]="0*100,34*30";
+w["awar"]="0*20,36*20";
+w["azard"]="0*20";
+w["ba"]="0*30";
+w["back"]="0*100";
 w["back-to-back"]="0*40";
-w["backtrac"]="0*929,2*130";
+w["backbarri"]="14*20";
+w["backtrac"]="0*899,2*100";
 w["backup"]="1*20";
 w["ballot"]="0*60";
 w["ballot_sync"]="0*30";
 w["bankruptci"]="1*10";
 w["bargain"]="1*10";
 w["barindex"]="14*60";
-w["barrier"]="0*776,2*100,13*10,14*1048";
+w["barrier"]="0*796,2*140,13*10,14*1228";
+w["barrier.cluster.arr"]="14*20";
+w["barrier.cluster.wait"]="14*20";
 w["barrierflag"]="14*60";
 w["base"]="0*200,1*10,14*20";
 w["bash"]="0*20";
-w["basi"]="1*30,35*30";
-w["basic"]="35*48";
-w["batch"]="3*90,14*440,34*158";
-w["batchmemopdata"]="3*168,5*60,13*10,14*260,34*30";
+w["basi"]="1*30,36*30";
+w["basic"]="36*48";
+w["batch"]="3*90,14*440,35*158";
+w["batchmemopdata"]="3*168,5*60,13*10,14*260,35*30";
 w["batchmemoptyp"]="3*40,14*80";
 w["bb"]="0*20";
-w["be"]="0*340,1*40,2*60,3*500,14*460,15*20,34*40,35*20";
+w["be"]="0*340,1*40,2*60,3*500,14*520,15*20,35*40,36*20";
 w["bear"]="0*20";
 w["becaus"]="0*20";
 w["becom"]="1*60";
-w["been"]="0*180,1*20,2*40,14*40,34*30,35*20";
-w["befor"]="0*330,1*20,2*20,34*100,35*40";
-w["beforehand"]="0*30";
-w["begin"]="3*220,14*260,34*30";
+w["been"]="0*180,1*20,2*40,14*40,35*30,36*20";
+w["befor"]="0*350,1*20,2*20,35*100,36*40";
+w["beforehand"]="0*50";
+w["begin"]="3*240,14*260,35*30";
 w["beginn"]="0*20";
 w["behalf"]="1*20";
-w["behav"]="0*40";
-w["behavior"]="0*315";
-w["believ"]="0*20,1*20,2*20,15*20,34*20,35*20";
+w["behav"]="0*40,35*20";
+w["behavior"]="0*315,35*30";
+w["believ"]="0*20,1*20,2*20,15*20,35*20,36*20";
 w["belong"]="0*40,14*20";
-w["below"]="0*380,1*30,34*70";
+w["below"]="0*400,1*30,35*70";
 w["beneficiari"]="1*10";
 w["benefit"]="1*20";
 w["beta"]="1*20";
-w["between"]="0*350,1*50,2*20,3*20,14*20,35*10";
+w["between"]="0*410,1*50,2*20,3*20,14*20,36*10";
 w["beyond"]="1*20";
-w["binari"]="0*40,1*80,2*10";
-w["binary-patch"]="0*20,2*10";
-w["bind"]="1*30,35*240";
+w["binari"]="0*20,1*80";
+w["bind"]="1*30,36*240";
 w["bio"]="1*10";
 w["bis"]="1*10";
 w["bit"]="0*150,2*20,14*40";
-w["ble"]="1*10";
-w["block"]="0*1595,2*20,14*180,35*30";
+w["bloc"]="0*30";
+w["block"]="0*1855,2*20,14*180,36*30";
 w["block-level"]="0*138";
 w["block_error"]="0*20";
 w["block_error.cu"]="0*47";
-w["blockdim"]="0*30,3*120,5*30,14*60";
+w["blockdim"]="3*120,5*30,14*60";
 w["blockdim_i"]="3*40,5*10,14*20";
 w["blockdim_x"]="3*40,5*10,14*20";
 w["blockdim_z"]="3*40,5*10,14*20";
 w["blockidx"]="0*70";
-w["blocksnb"]="35*30";
-w["board"]="0*20,1*20,2*20,15*20,34*20,35*20";
+w["blocksnb"]="36*30";
+w["board"]="0*20,1*20,2*20,15*20,35*20,36*20";
 w["bodi"]="1*10";
-w["boolean"]="3*100,14*100";
-w["both"]="0*100,1*20,35*30";
-w["bound"]="0*590,1*10,35*150";
+w["boolean"]="3*120,14*120";
+w["both"]="0*140,1*20,36*30";
+w["bound"]="0*590,1*10,2*20,36*150";
 w["brace"]="0*20";
-w["branch"]="35*20";
+w["branch"]="2*20,36*20";
 w["breach"]="1*90";
 w["brief"]="3*10";
 w["broader"]="1*10";
 w["bufer"]="0*60";
-w["buffer"]="0*260,1*20,14*140";
-w["bug"]="2*40";
-w["build"]="0*166,2*20";
-w["built"]="0*100,2*10,14*20";
+w["buffer"]="0*260,1*20,14*160";
+w["bug"]="2*60";
+w["build"]="0*166,2*40";
+w["built"]="0*100,14*20";
 w["bundl"]="1*10";
 w["bureau"]="1*10";
 w["busi"]="1*80";
 w["business-crit"]="1*10";
 w["bypass"]="1*10";
-w["byte"]="0*990,3*20,14*220,35*120";
+w["byte"]="0*1030,3*20,14*260,36*120";
 w["c,fortran"]="0*20";
 w["c,launch-count"]="0*20";
 w["ca"]="1*10";
-w["cach"]="0*40,2*40,14*200";
+w["cach"]="0*40,2*40,14*220";
 w["cachecontrolinstructionkind"]="14*100";
 w["california"]="1*20";
-w["call"]="0*1200,2*70,3*80,14*1060,34*440,35*140";
+w["call"]="0*1200,2*70,3*80,14*1040,35*480,36*140";
 w["callackidsync"]="14*40";
-w["callback"]="2*260,3*2784,13*10,14*4848,34*2374";
-w["callback_api"]="14*20";
-w["callbackdata"]="3*188,5*60,13*10,14*260,34*120";
-w["callbackdomain"]="14*200,34*80";
-w["callbackfunc"]="3*360,14*80,34*20";
-w["callbacki"]="14*20";
-w["callbackid"]="14*160,34*60";
+w["callback"]="2*260,3*2952,13*10,14*5228,35*2504";
+w["callbackdata"]="3*188,5*60,13*10,14*260,35*120";
+w["callbackdomain"]="14*200,35*80";
+w["callbackfunc"]="3*380,14*80,35*20";
+w["callbackid"]="14*160,35*60";
 w["callbackidbatchmemop"]="14*40";
 w["callbackidev"]="14*40";
+w["callbackidexternalmemori"]="14*40";
 w["callbackidgraph"]="14*40";
 w["callbackidlaunch"]="14*40";
 w["callbackidmemcpi"]="14*40";
-w["callbackidmemset"]="14*20";
+w["callbackidmemset"]="14*40";
 w["callbackidresourc"]="14*40";
 w["callbackiduvm"]="14*40";
 w["callbacks"]="14*20";
-w["callbacksit"]="3*40,5*10,34*60";
+w["callbacksit"]="3*40,5*10,35*60";
 w["caller"]="0*80,2*60";
 w["callflag"]="14*40";
 w["callsit"]="0*60";
-w["can"]="0*1690,1*20,2*40,3*100,14*680,34*590,35*580";
+w["can"]="0*1890,1*20,2*30,3*120,14*900,35*650,36*580";
 w["canari"]="2*40";
 w["candid"]="0*20";
-w["cannot"]="0*60,1*10,2*10,3*20,14*120,35*30";
+w["cannot"]="0*60,1*10,2*10,3*20,14*120,36*30";
 w["canon"]="0*20";
 w["capabl"]="0*90,2*20";
 w["capit"]="1*10";
 w["captur"]="0*40";
-w["case"]="0*731,1*10,2*50,34*30";
-w["cast"]="34*30";
+w["case"]="0*771,1*10,2*70,35*88";
+w["cast"]="35*30";
 w["catastroph"]="1*10";
-w["caus"]="0*410,1*80,2*10,34*70,35*30";
-w["cb"]="3*380,14*700,34*300";
-w["cbdata"]="3*720,14*200,34*320";
-w["cbid"]="3*1300,14*2360,34*310";
-w["cbinfo"]="34*90";
+w["caus"]="0*470,1*80,2*10,35*70,36*30";
+w["cb"]="3*400,14*680,35*300";
+w["cbdata"]="3*760,14*200,35*320";
+w["cbid"]="3*1400,14*2600,35*310";
+w["cbinfo"]="35*90";
 w["cbname"]="14*120";
 w["cbsite"]="14*20";
-w["ccheck"]="2*20";
 w["cdp"]="0*20";
+w["ce"]="1*10";
 w["ceas"]="1*10";
-w["certain"]="1*40,34*60";
+w["certain"]="1*40,35*60";
 w["certifi"]="1*20";
-w["cg"]="14*20";
-w["ch"]="5*10";
-w["chang"]="0*40,1*40,2*20,3*40,14*60,15*20,34*20,35*80";
+w["cg"]="14*60";
+w["chang"]="0*40,1*40,2*20,3*40,14*60,15*20,35*20,36*80";
 w["char"]="0*390,3*160,14*360";
-w["charact"]="0*60,35*20";
+w["charact"]="0*60,36*20";
 w["charg"]="1*20";
-w["check"]="0*1322,1*10,2*80,34*90,35*30";
+w["check"]="0*1512,1*10,2*90,35*120,36*30";
 w["check-api-memory-access"]="0*20";
 w["check-cache-control"]="0*20,2*20";
-w["check-device-heap"]="0*40,2*30";
+w["check-device-heap"]="0*40,2*20";
 w["check-exit-cod"]="0*20";
+w["check-optix"]="0*40,2*20";
 w["check-optix-leak"]="0*40";
-w["checker"]="34*20";
-w["child"]="0*60,2*40";
+w["check-warpgroup-mm"]="0*20";
+w["check-warpgroup-mma"]="0*20";
+w["check/memcheck_demo"]="0*30";
+w["checker"]="35*20";
+w["child"]="0*60,2*60";
 w["children"]="0*20";
 w["chip"]="0*60";
 w["choic"]="0*20";
@@ -623,79 +648,84 @@ w["chosen"]="0*30";
 w["chronolog"]="0*20";
 w["circumv"]="1*10";
 w["citizen"]="1*10";
+w["cktrace"]="0*30";
 w["claim"]="1*50";
 w["clara"]="1*30";
 w["clariti"]="0*30";
 w["class"]="0*40,14*37";
 w["classif"]="0*40";
 w["claus"]="1*10";
-w["clear"]="0*0,1*0,2*0,3*0,5*0,13*0,14*0,15*0,34*0,35*0";
-w["cli"]="2*20";
+w["clear"]="0*0,1*0,2*0,3*0,5*0,13*0,14*0,15*0,35*0,36*0";
+w["cli"]="2*40";
 w["click"]="1*10";
 w["client"]="14*20";
-w["cluster"]="2*20";
-w["cluster.sync"]="2*20";
+w["cluster"]="0*238,2*60,14*220";
+w["cluster-wid"]="0*60";
+w["cluster.sync"]="0*20,2*20";
 w["clusterdim"]="3*120,5*30,14*60";
 w["clusterdim_i"]="3*40,5*10,14*20";
 w["clusterdim_x"]="3*40,5*10,14*20";
 w["clusterdim_z"]="3*40,5*10,14*20";
 w["cnp"]="14*20";
-w["code"]="0*525,1*160,2*60,3*20,13*10,14*228,34*415,35*30";
+w["co"]="3*20,14*20";
+w["code"]="0*545,1*160,2*60,3*20,13*10,14*228,35*445,36*30";
 w["collect"]="1*50";
 w["column"]="0*80";
-w["com"]="0*20,1*30,35*30";
-w["combin"]="1*10,14*60,35*80";
+w["com"]="0*40,1*30,36*30";
+w["combin"]="1*10,14*80,36*80";
 w["comma"]="0*60";
-w["command"]="0*248,2*110";
+w["command"]="0*268,2*110";
 w["command-lin"]="2*80";
 w["commenc"]="1*10";
-w["comment"]="0*40,1*10,34*30";
+w["comment"]="0*40,1*10,35*30";
 w["commerci"]="1*70";
-w["commit"]="0*40,1*30,14*140";
+w["commit"]="0*40,1*30,14*180";
 w["commit/wait"]="0*20";
 w["common"]="0*50,1*10";
-w["communic"]="0*40,14*220";
-w["compani"]="0*40,1*80,2*40,15*40,34*40,35*40";
+w["communic"]="0*40,14*240";
+w["compani"]="0*40,1*80,2*40,15*40,35*40,36*40";
 w["compar"]="0*30";
-w["compat"]="14*40,34*20,35*38";
-w["compatibili"]="34*38";
-w["compatid"]="35*230";
+w["compat"]="14*40,35*58,36*38";
+w["compatid"]="36*230";
+w["compens"]="35*30";
 w["compet"]="1*20";
-w["compil"]="0*498,1*30,2*40,34*60";
-w["compile-as-tools-patch"]="34*30";
-w["complet"]="0*160,2*10,14*220";
-w["compli"]="1*20,35*10";
+w["compil"]="0*498,1*30,2*40,35*60";
+w["compile-as-tools-patch"]="35*30";
+w["complet"]="0*160,2*10,14*240";
+w["compli"]="1*20,36*10";
 w["complianc"]="1*10";
-w["compon"]="0*100,1*70,2*20,15*20,34*20,35*20";
-w["compos"]="0*20,34*20";
+w["compon"]="0*80,1*70,2*20,15*20,35*20,36*20";
+w["componen"]="0*20";
+w["compos"]="0*20,35*20";
 w["composit"]="1*10";
-w["comput"]="0*4536,1*131,2*311,3*31,5*31,13*51,14*31,15*31,34*451,35*501";
+w["comput"]="0*4596,1*131,2*281,3*31,5*31,13*51,14*31,15*31,35*501,36*501";
 w["compute-sanit"]="0*640,2*20";
-w["concurr"]="0*50,14*180";
-w["condit"]="0*230,1*220,34*20";
-w["confidenti"]="1*238";
-w["confidentia"]="1*10";
-w["configur"]="1*10,2*10,3*40,14*40";
+w["compute-sanitizer-samples/memcheck/memcheck_demo"]="0*30";
+w["concurr"]="0*70,14*180";
+w["condit"]="0*230,1*220,35*20";
+w["confidenti"]="1*248";
+w["configur"]="1*10,3*40,14*40";
 w["confirm"]="1*10";
 w["conflict"]="0*20,1*30";
 w["conjunct"]="0*20";
 w["connect"]="0*160,1*60,14*20";
 w["consent"]="1*60";
-w["consequ"]="0*20,1*20,2*20,15*20,34*20,35*20";
+w["consequ"]="0*20,1*20,2*20,15*20,35*20,36*20";
 w["consequenti"]="1*50";
-w["consid"]="0*60,35*30";
+w["consid"]="0*60,36*30";
 w["consist"]="0*40,1*20";
 w["consolid"]="1*10";
-w["const"]="0*210,3*280,14*760,34*60";
+w["const"]="0*210,3*280,14*780,35*60";
 w["constant"]="3*40,14*20";
 w["constitut"]="1*40";
 w["constrain"]="0*20";
 w["constru"]="1*20";
 w["consum"]="0*20,1*10,14*20";
-w["contain"]="0*840,1*30,3*240,14*560,34*60,35*20";
+w["contain"]="0*840,1*30,3*260,14*600,35*90,36*20";
+w["containsdevicegraphlaunch"]="3*40,5*10,14*40";
 w["content"]="0*60,1*10,13*30";
 w["contest"]="1*10";
-w["context"]="0*380,1*10,3*1270,5*10,14*1060,34*210";
+w["context"]="0*380,1*10,2*20,3*1350,5*10,14*1080,35*210";
 w["context,kernel"]="0*20";
 w["contigu"]="0*20";
 w["continu"]="0*210,1*30";
@@ -707,29 +737,30 @@ w["contrari"]="1*10";
 w["contribut"]="0*20";
 w["contributor"]="1*60";
 w["control"]="0*260,1*100,2*80,14*220";
-w["conveni"]="35*90";
+w["conveni"]="36*90";
 w["convent"]="1*10";
 w["converg"]="0*90";
 w["convers"]="1*10";
 w["convey"]="0*20";
-w["cooper"]="0*40,2*40,14*20";
+w["cooper"]="0*40,2*60,14*20";
 w["coordin"]="0*20";
-w["copi"]="0*138,1*120,3*380,14*220,34*40";
-w["copyright"]="0*38,1*398,2*38,15*38,34*38,35*38";
+w["copi"]="0*138,1*120,3*400,14*220,35*40";
+w["copyright"]="0*38,1*398,2*38,15*38,35*38,36*38";
 w["core"]="0*45";
 w["core.name.nvcudmp"]="0*25";
-w["coredump"]="0*218,2*50";
-w["corpor"]="0*120,1*150,2*120,15*120,34*120,35*120";
-w["correct"]="0*80,1*10,14*20,34*20";
-w["correspond"]="0*100,1*10,2*20,14*40,34*50";
+w["coredump"]="0*318,2*110";
+w["coredump-nam"]="0*40,2*20";
+w["corpor"]="0*120,1*150,2*120,15*120,35*120,36*120";
+w["correct"]="0*80,1*10,14*20,35*20";
+w["correspond"]="0*100,1*10,2*20,14*40,35*50";
 w["corrupt"]="0*40";
 w["cost"]="1*40";
-w["could"]="0*20,2*20,35*20";
-w["count"]="0*60,2*30,14*320,34*60";
-w["counter"]="0*20,14*340";
+w["could"]="0*20,2*20,36*20";
+w["count"]="0*60,2*30,14*320,35*60";
+w["counter"]="0*20,14*480";
 w["counterpart"]="0*20";
 w["counti"]="1*10";
-w["countri"]="0*20,1*40,2*20,15*20,34*20,35*20";
+w["countri"]="0*20,1*40,2*20,15*20,35*20,36*20";
 w["cours"]="1*20";
 w["court"]="1*30";
 w["cover"]="1*30";
@@ -739,102 +770,109 @@ w["cp.async.commit_group"]="14*20";
 w["cp.async.wait_al"]="14*20";
 w["cp.async.wait_group"]="14*20";
 w["cpp"]="0*30";
-w["cpu"]="0*20,35*60";
-w["crash"]="2*60";
-w["creat"]="0*230,1*10,3*380,14*480,34*60,35*290";
-w["creation"]="3*70,14*80,34*110,35*30";
+w["cpu"]="0*20,36*60";
+w["crash"]="0*20,2*80";
+w["creat"]="0*230,1*10,3*420,14*520,35*60,36*290";
+w["creation"]="3*70,14*80,35*110,36*30";
 w["credit"]="1*10";
 w["creditor"]="1*10";
-w["critic"]="0*20,1*70,2*20,15*20,34*20,35*20";
+w["critic"]="0*20,1*70,2*20,15*20,35*20,36*20";
 w["csu"]="0*150";
-w["ction"]="14*20";
 w["ctrl"]="2*10";
 w["ctrl-c"]="2*10";
-w["ctx"]="14*540,34*60";
-w["cu"]="0*735,3*100,14*160,34*30";
+w["ctx"]="14*540,35*60";
+w["cu"]="0*775,3*100,14*160,35*30";
 w["cu_graph_node_type_kernel"]="3*20";
+w["cu_graph_node_type_mem"]="3*20,14*20";
 w["cu_graph_node_type_mem_alloc"]="3*20,14*40";
-w["cu_graph_node_type_mem_fre"]="3*20,14*40";
+w["cu_graph_node_type_mem_fre"]="14*20";
 w["cu_graph_node_type_memcpi"]="3*20,14*40";
-w["cu_graph_node_type_memset"]="3*20,14*40";
+w["cu_graph_node_type_memset"]="14*40";
 w["cuarray"]="3*40,14*40";
-w["cubin"]="2*20,3*40,14*40,34*150";
+w["cubin"]="2*20,3*40,14*40,35*150";
 w["cubins"]="3*40,5*10,14*40";
-w["cucontext"]="3*680,14*520,34*30";
-w["cuctxsynchron"]="34*30";
-w["cuda"]="0*3411,2*580,3*280,14*1560,34*1400,35*340";
-w["cuda-gdb"]="0*60";
+w["cubla"]="2*20";
+w["cucontext"]="3*760,14*560,35*30";
+w["cuctxsynchron"]="35*30";
+w["cud"]="0*50";
+w["cuda"]="0*3421,2*670,3*280,14*1640,35*1490,36*340";
+w["cuda-gdb"]="0*80";
 w["cuda/direct3d"]="2*10";
 w["cuda/vulkan"]="2*10";
 w["cuda::barri"]="0*196,2*40";
+w["cuda::barrier::wait"]="2*20";
 w["cuda::memcpy_async"]="0*20,2*20";
 w["cuda_error_not_found"]="0*20,2*20";
 w["cudabarrierinstructionkind"]="14*80";
 w["cudacor"]="0*25";
+w["cudadev"]="0*30";
 w["cudadevicedisablepeeraccess"]="0*20";
 w["cudadeviceenablepeeraccess"]="0*20";
 w["cudadevicereset"]="2*20";
-w["cudadevicesynchron"]="0*150,34*30";
+w["cudadevicesynchron"]="0*120,35*30";
 w["cudaerror"]="0*120,3*20,14*20";
 w["cudaerror_t"]="3*20,14*20";
+w["cudaerrorlaunchfailur"]="0*20";
 w["cudaerrornotreadi"]="0*20";
 w["cudaerrorpeeraccessalreadyen"]="0*20";
 w["cudaerrorpeeraccessnoten"]="0*20";
 w["cudaeventqueri"]="0*20";
-w["cudafre"]="0*290,14*20,34*40,35*80";
+w["cudafre"]="0*290,14*20,35*40,36*80";
 w["cudafreeasync"]="0*40,14*40";
 w["cudafreehost"]="14*20";
+w["cudagetcurrentgraphexec"]="35*30";
 w["cudahostalloc"]="0*20,14*20";
 w["cudahostregist"]="0*20";
 w["cudalaunchkernel"]="0*240";
-w["cudamalloc"]="0*490,14*60,34*50,35*60";
+w["cudamalloc"]="0*490,14*60,35*50,36*60";
 w["cudamallocasync"]="0*20";
 w["cudamallochost"]="0*20,14*20";
 w["cudamallocmanag"]="14*20";
 w["cudamemattachglob"]="14*20";
 w["cudamemattachhost"]="14*20";
 w["cudamemattachsingl"]="14*40";
-w["cudamemcpi"]="0*40,2*10,14*20,34*260,35*10";
+w["cudamemcpi"]="0*40,2*30,14*20,35*260,36*10";
 w["cudamemcpy/cudamemset"]="0*20";
-w["cudamemcpy_v3020_param"]="34*60";
-w["cudamemcpyasync"]="14*20,34*20";
-w["cudamemcpykind"]="34*30";
-w["cudamemset"]="0*170,14*20,34*50,35*10";
+w["cudamemcpy_v3020_param"]="35*60";
+w["cudamemcpyasync"]="14*20,35*20";
+w["cudamemcpykind"]="35*30";
+w["cudamemset"]="0*170,14*20,35*50,36*10";
 w["cudart"]="0*180";
-w["cudastreamattachmemasync"]="2*20,14*20";
+w["cudastreamattac"]="14*20";
+w["cudastreamattachmemasync"]="2*20";
 w["cudastreamqueri"]="0*20";
-w["cudastreamsynchron"]="14*20,34*30";
-w["cudevic"]="3*240,14*240";
+w["cudastreamsynchron"]="14*20,35*30";
+w["cudevic"]="3*280,14*280";
 w["cuevent"]="3*40";
-w["cufunct"]="3*80,14*240,34*60";
+w["cuexternalmemori"]="3*40,14*40";
+w["cufunct"]="3*80,14*240,35*60";
 w["cugetprocaddress"]="0*20,2*20";
 w["cugraph"]="3*40";
-w["cugraphexec"]="3*120,14*80";
+w["cugraphexec"]="3*120,14*120";
 w["cugraphnod"]="3*40,14*40";
 w["cugraphnodetyp"]="3*40,14*40";
 w["culibrari"]="3*40,14*40";
-w["cumemalloc"]="35*30";
+w["cumemalloc"]="36*30";
 w["cumemorypool"]="3*80,14*40";
 w["cumemsetaccess"]="2*20";
-w["cumodul"]="3*120,14*280,34*30";
-w["cumoduleload"]="14*20,34*30";
-w["cumoduleloaddata"]="14*20,34*30";
+w["cumodul"]="3*120,14*280,35*30";
+w["cumoduleload"]="14*20,35*30";
+w["cumoduleloaddata"]="14*20,35*30";
 w["cumoduleloadfatbinari"]="14*20";
 w["cumul"]="1*20";
 w["cure"]="1*10";
 w["curesult"]="3*20,14*20";
-w["current"]="0*230,1*10,2*10,3*40,14*320,34*30,35*30";
+w["current"]="0*230,1*10,2*10,3*40,14*320,35*30,36*30";
 w["currentcontrolset"]="0*20";
-w["custom"]="1*10,35*30";
+w["custom"]="1*10,36*30";
 w["custream"]="0*120,3*480,14*700";
 w["custream_st"]="0*120";
-w["custreamsynchron"]="34*30";
-w["custreamwritevalu"]="34*30";
-w["cwarp"]="14*20";
+w["custreamsynchron"]="35*30";
+w["custreamwritevalu"]="35*30";
 w["d_vec"]="0*90";
 w["da"]="0*30";
 w["damag"]="1*150";
-w["data"]="0*1210,1*100,2*70,3*1679,5*29,13*20,14*1280,34*225";
+w["data"]="0*1230,1*100,2*70,3*1769,5*29,13*20,14*1480,35*315";
 w["data_block"]="0*120";
 w["data_in"]="0*390";
 w["databas"]="1*10";
@@ -844,31 +882,30 @@ w["day"]="1*30";
 w["dbffbd"]="0*20";
 w["dbg"]="0*30";
 w["dd"]="0*30";
-w["de"]="2*20,34*30";
-w["de-alloc"]="34*30";
+w["de"]="35*60";
+w["de-alloc"]="35*30";
 w["deal"]="0*20,1*20";
 w["death"]="1*10";
+w["deb"]="0*20";
 w["debt"]="1*10";
-w["debug"]="0*343,1*10,2*20";
-w["debuglevel"]="0*20";
-w["dec"]="1*10";
+w["debug"]="0*363,1*10,2*20";
 w["decis"]="1*10";
 w["declar"]="0*110";
 w["decod"]="1*33";
+w["decompi"]="1*10";
 w["decreas"]="0*20";
-w["deddata"]="14*20";
-w["deddata::funct"]="14*20";
 w["deem"]="1*10";
-w["default"]="0*520,35*50";
+w["default"]="0*580,36*50";
 w["default/valu"]="0*20";
 w["defect"]="1*10";
 w["defend"]="1*10";
-w["defin"]="0*290,1*10,34*130";
+w["defin"]="0*290,1*10,35*130";
 w["definit"]="3*20,14*20";
 w["delawar"]="1*10";
 w["delay"]="1*20";
 w["deleg"]="1*10";
-w["deliv"]="1*40,34*20";
+w["delimit"]="14*20";
+w["deliv"]="1*40,35*20";
 w["deliveri"]="1*10";
 w["delta"]="0*60";
 w["deltapi"]="0*20";
@@ -877,132 +914,144 @@ w["demo"]="0*1489";
 w["demonstr"]="0*30";
 w["dens"]="0*20";
 w["depart"]="1*20";
-w["depend"]="0*220,14*80";
+w["depend"]="0*200,14*80";
 w["derefer"]="3*20";
+w["dereferenc"]="35*30";
 w["deriv"]="1*80";
-w["describ"]="0*60,1*10,14*160,34*20";
-w["descript"]="0*140,3*10,14*1040";
-w["design"]="0*40,1*70,2*20,15*20,34*20,35*20";
-w["destin"]="3*100,14*180,34*20";
-w["destroy"]="0*310,1*20,3*400,14*500,35*30";
-w["destroy-on-device-error"]="0*90";
-w["destruct"]="34*90,35*30";
-w["detail"]="0*160,2*50,3*20,14*20,34*20";
-w["detect"]="0*1364,2*100,14*20,34*30";
+w["describ"]="0*60,1*10,14*200,35*20";
+w["descript"]="0*160,3*10,14*1075";
+w["design"]="0*40,1*70,2*20,15*20,35*20,36*20";
+w["destin"]="3*100,14*200";
+w["destr"]="0*20";
+w["destroy"]="0*290,1*20,3*400,14*540,36*30";
+w["destroy-on-device-error"]="0*70";
+w["destruct"]="35*90,36*30";
+w["detail"]="0*160,2*50,3*20,14*40,35*20";
+w["detect"]="0*1602,2*160,14*20,35*30";
 w["detector"]="0*40";
-w["determin"]="0*20,1*10,34*30";
+w["determin"]="0*20,1*10,35*30";
 w["determinist"]="0*40";
 w["detour"]="1*78";
-w["dev"]="35*30";
-w["develop"]="0*100,1*123,2*30,15*20,34*20,35*20";
-w["devic"]="0*1962,1*30,2*300,3*400,5*10,14*1720,15*20,34*180,35*370";
+w["dev"]="36*30";
+w["develop"]="0*100,1*123,2*30,15*20,35*20,36*20";
+w["devic"]="0*1982,1*30,2*310,3*520,5*10,14*1980,15*20,35*528,36*370";
 w["device-debug"]="2*20";
-w["device-sid"]="2*20,14*200,34*20";
-w["device-to-host"]="2*10,34*20";
+w["device-launch"]="14*60";
+w["device-sid"]="2*20,14*200,35*20";
+w["device-to-host"]="2*10,35*20";
 w["devicecallbacknam"]="14*180";
-w["devicedatatrack"]="34*120";
+w["devicedatatrack"]="35*120";
+w["devicegraphlaunchescontext"]="3*40,5*10,14*40";
 w["devicememoryflag"]="14*80";
-w["devicepermhandl"]="35*60";
+w["devicepermhandl"]="36*60";
 w["devis"]="1*10";
 w["devptr"]="14*300";
 w["dfar"]="1*10";
 w["dfd"]="0*150";
-w["diagnost"]="0*20,1*20,2*20,15*20,34*20,35*20";
+w["diagnost"]="0*20,1*20,2*20,15*20,35*20,36*20";
 w["did"]="0*40";
-w["differ"]="0*310,1*20,3*20,14*20";
+w["didn"]="0*40";
+w["didn't"]="0*40";
+w["differ"]="0*310,1*20,3*20,14*20,35*80";
 w["digit"]="1*10";
 w["dim"]="0*240";
 w["dim3"]="0*240";
-w["direct"]="0*20,1*70,2*40,3*60,5*10,14*260,34*30";
+w["direct"]="0*20,1*70,2*40,3*60,5*10,14*260,35*30";
 w["director"]="1*20";
-w["disabl"]="0*100,1*10,2*50,3*40,14*340";
+w["disabl"]="0*120,1*10,2*40,3*40,14*340";
 w["disassembl"]="1*10";
-w["disclaim"]="0*20,1*170,2*20,15*20,34*20,35*20";
+w["disclaim"]="0*20,1*170,2*20,15*20,35*20,36*20";
 w["disclos"]="1*80";
 w["disclosur"]="1*70";
 w["discontinu"]="1*10";
 w["discret"]="1*30";
 w["discuss"]="1*10";
 w["dismiss"]="1*10";
-w["display"]="0*260";
-w["dispos"]="35*30";
+w["display"]="0*280";
+w["dispos"]="36*30";
 w["dissolut"]="1*10";
 w["distinct"]="2*20";
-w["distribut"]="1*120,2*30";
+w["distribut"]="0*20,1*120,2*40";
 w["distributor"]="1*10";
 w["diverg"]="0*315,2*40";
 w["divergent_thread"]="0*20";
 w["divergent_threads.cu"]="0*47";
 w["dma"]="14*40";
-w["dmemset"]="14*20";
-w["do"]="0*80,1*50,2*90,34*20";
-w["document"]="0*72,1*215,2*92,3*32,5*52,13*32,14*92,15*52,34*82,35*52";
-w["doe"]="0*250,1*60,2*10,14*60,35*20";
-w["doma"]="3*20";
-w["domain"]="0*20,3*720,14*2220,34*925,35*98";
-w["don"]="1*20,2*10";
-w["don't"]="2*10";
+w["do"]="0*80,1*50,2*70,35*20";
+w["document"]="0*132,1*215,2*112,3*32,5*52,13*32,14*112,15*52,35*82,36*52";
+w["doe"]="0*250,1*60,2*20,14*60,36*20";
+w["doesn"]="3*20,14*20";
+w["doesn't"]="3*20,14*20";
+w["domain"]="0*20,3*780,14*2260,35*905,36*98";
+w["don"]="1*20";
 w["done"]="3*20,14*20";
 w["don’t"]="1*20";
 w["doubl"]="0*60";
-w["download"]="1*70,35*20";
-w["draw"]="0*20,1*20,2*20,15*20,34*20,35*20";
+w["download"]="1*70,36*20";
+w["draw"]="0*20,1*20,2*20,15*20,35*20,36*20";
 w["dress"]="1*10";
 w["drive"]="0*20";
-w["driver"]="0*250,1*10,2*40,3*270,14*480,34*248";
+w["driver"]="0*250,1*10,2*60,3*310,14*560,35*248";
 w["drop"]="14*80";
 w["dst"]="14*180";
 w["dstaddress"]="3*40,5*10,14*40";
 w["dstcontext"]="3*40,5*10,14*40";
-w["dstpit"]="5*10";
-w["dstpitch"]="3*40,14*40";
+w["dstpitch"]="3*40,5*10,14*40";
 w["dststream"]="3*40,5*10,14*40";
-w["due"]="0*120,1*30,2*10,14*20";
+w["due"]="0*140,1*30,2*10,14*20";
+w["dump"]="0*20";
 w["duplic"]="1*10";
 w["durat"]="1*20";
-w["dure"]="0*30,1*50,2*10";
+w["dure"]="0*30,1*50,2*10,35*30";
 w["dword"]="0*20";
-w["dynam"]="0*118,2*30,14*20,34*20";
-w["e.g"]="0*60,35*60";
-w["e_async_don"]="14*20";
-w["each"]="0*500,1*100,3*20,5*10,14*80,34*110";
-w["ead"]="2*20";
+w["dynam"]="0*118,2*30,14*20,35*20";
+w["e.g"]="0*60,36*60";
+w["each"]="0*520,1*100,3*20,5*10,14*80,35*140";
 w["ear"]="1*10";
-w["earli"]="0*100";
+w["earli"]="0*180";
 w["earlier"]="0*40,1*40";
-w["easi"]="34*30";
-w["easier"]="34*20";
+w["easi"]="35*30";
+w["easier"]="35*20";
 w["easili"]="0*20";
 w["ec"]="0*120";
+w["ecial"]="1*20";
 w["econom"]="1*30";
+w["ed"]="0*50,1*20,14*40";
 w["edec"]="0*20";
 w["edf"]="0*20";
 w["educ"]="1*43";
 w["educational/academ"]="1*23";
 w["ef"]="0*30";
-w["effect"]="0*60,1*83";
+w["effect"]="0*60,1*73";
 w["effort"]="1*30";
-w["either"]="0*60,1*20,34*20";
-w["element"]="0*420,35*20";
+w["either"]="0*100,1*20,14*60,35*20";
+w["el"]="0*20";
+w["element"]="0*390,36*20";
 w["elimin"]="0*40";
-w["els"]="34*30";
+w["els"]="35*30";
 w["elsewher"]="1*10";
 w["email"]="1*10";
 w["embargo"]="1*10";
 w["employ"]="1*10";
 w["employe"]="1*60";
-w["empti"]="0*20,2*20,14*80,35*40";
-w["en"]="0*20,1*20";
-w["enabl"]="0*510,1*10,2*110,3*40,14*780,34*20,35*20";
+w["empti"]="0*20,2*20,14*100,36*40";
+w["en"]="0*20,1*30";
+w["enabl"]="0*550,1*10,2*140,3*40,14*780,35*20,36*20";
 w["enabled/dis"]="14*40";
-w["encod"]="1*33,35*10";
-w["encount"]="0*230";
+w["encod"]="1*33,36*10";
+w["encount"]="0*290";
 w["encourag"]="1*10";
 w["encrypt"]="1*10";
 w["end"]="0*20,1*30,2*10,3*60,14*80";
 w["endors"]="1*50";
 w["enforc"]="1*10";
-w["engag"]="1*10";
 w["engin"]="1*10";
 w["enhanc"]="1*10";
+w["enlarg"]="1*10";
+w["enough"]="0*20,14*20";
+w["enrol"]="1*10";
+w["ensur"]="0*130,14*20,35*50";
+w["ent"]="1*10";
+w["enter"]="0*20,1*40,14*100,35*30";
+w["enterpris"]="1*196";
 
