@@ -98,6 +98,12 @@ typedef struct cuDeviceGetDefaultMemPool_params_st {
     CUdevice dev;
 } cuDeviceGetDefaultMemPool_params;
 
+typedef struct cuDeviceGetExecAffinitySupport_params_st {
+    int *pi;
+    CUexecAffinityType type;
+    CUdevice dev;
+} cuDeviceGetExecAffinitySupport_params;
+
 typedef struct cuFlushGPUDirectRDMAWrites_params_st {
     CUflushGPUDirectRDMAWritesTarget target;
     CUflushGPUDirectRDMAWritesScope scope;
@@ -138,12 +144,6 @@ typedef struct cuDevicePrimaryCtxReset_v2_params_st {
     CUdevice dev;
 } cuDevicePrimaryCtxReset_v2_params;
 
-typedef struct cuDeviceGetExecAffinitySupport_params_st {
-    int *pi;
-    CUexecAffinityType type;
-    CUdevice dev;
-} cuDeviceGetExecAffinitySupport_params;
-
 typedef struct cuCtxCreate_v2_params_st {
     CUcontext *pctx;
     unsigned int flags;
@@ -263,6 +263,10 @@ typedef struct cuModuleUnload_params_st {
     CUmodule hmod;
 } cuModuleUnload_params;
 
+typedef struct cuModuleGetLoadingMode_params_st {
+    CUmoduleLoadingMode *mode;
+} cuModuleGetLoadingMode_params;
+
 typedef struct cuModuleGetFunction_params_st {
     CUfunction *hfunc;
     CUmodule hmod;
@@ -1262,6 +1266,13 @@ typedef struct cuLaunchKernel_ptsz_params_st {
     void **extra;
 } cuLaunchKernel_ptsz_params;
 
+typedef struct cuLaunchKernelEx_ptsz_params_st {
+    const CUlaunchConfig *config;
+    CUfunction f;
+    void **kernelParams;
+    void **extra;
+} cuLaunchKernelEx_ptsz_params;
+
 typedef struct cuLaunchCooperativeKernel_ptsz_params_st {
     CUfunction f;
     unsigned int gridDimX;
@@ -1850,6 +1861,18 @@ typedef struct cuOccupancyAvailableDynamicSMemPerBlock_params_st {
     int blockSize;
 } cuOccupancyAvailableDynamicSMemPerBlock_params;
 
+typedef struct cuOccupancyMaxPotentialClusterSize_params_st {
+    int *clusterSize;
+    CUfunction func;
+    const CUlaunchConfig *config;
+} cuOccupancyMaxPotentialClusterSize_params;
+
+typedef struct cuOccupancyMaxActiveClusters_params_st {
+    int *numClusters;
+    CUfunction func;
+    const CUlaunchConfig *config;
+} cuOccupancyMaxActiveClusters_params;
+
 typedef struct cuTexRefSetArray_params_st {
     CUtexref hTexRef;
     CUarray hArray;
@@ -2107,6 +2130,30 @@ typedef struct cuGraphicsUnmapResources_ptsz_params_st {
     CUstream hStream;
 } cuGraphicsUnmapResources_ptsz_params;
 
+typedef struct cuCoredumpGetAttribute_params_st {
+    CUcoredumpSettings attrib;
+    void *value;
+    size_t *size;
+} cuCoredumpGetAttribute_params;
+
+typedef struct cuCoredumpGetAttributeGlobal_params_st {
+    CUcoredumpSettings attrib;
+    void *value;
+    size_t *size;
+} cuCoredumpGetAttributeGlobal_params;
+
+typedef struct cuCoredumpSetAttribute_params_st {
+    CUcoredumpSettings attrib;
+    void *value;
+    size_t *size;
+} cuCoredumpSetAttribute_params;
+
+typedef struct cuCoredumpSetAttributeGlobal_params_st {
+    CUcoredumpSettings attrib;
+    void *value;
+    size_t *size;
+} cuCoredumpSetAttributeGlobal_params;
+
 typedef struct cuGetExportTable_params_st {
     const void **ppExportTable;
     const CUuuid *pExportTableId;
@@ -2757,6 +2804,13 @@ typedef struct cuLaunchKernel_params_st {
     void **extra;
 } cuLaunchKernel_params;
 
+typedef struct cuLaunchKernelEx_params_st {
+    const CUlaunchConfig *config;
+    CUfunction f;
+    void **kernelParams;
+    void **extra;
+} cuLaunchKernelEx_params;
+
 typedef struct cuLaunchHostFunc_params_st {
     CUstream hStream;
     CUhostFn fn;
