@@ -387,6 +387,12 @@ typedef struct cudaLaunchKernel_ptsz_v7000_params_st {
     cudaStream_t stream;
 } cudaLaunchKernel_ptsz_v7000_params;
 
+typedef struct cudaLaunchKernelExC_ptsz_v11060_params_st {
+    const cudaLaunchConfig_t *config;
+    const void *func;
+    void **args;
+} cudaLaunchKernelExC_ptsz_v11060_params;
+
 typedef struct cudaLaunchCooperativeKernel_ptsz_v9000_params_st {
     const void *func;
     dim3 gridDim;
@@ -459,6 +465,18 @@ typedef struct cudaOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_v7000_para
     unsigned int flags;
 } cudaOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_v7000_params;
 
+typedef struct cudaOccupancyMaxPotentialClusterSize_v11070_params_st {
+    int *clusterSize;
+    const void *func;
+    const cudaLaunchConfig_t *launchConfig;
+} cudaOccupancyMaxPotentialClusterSize_v11070_params;
+
+typedef struct cudaOccupancyMaxActiveClusters_v11070_params_st {
+    int *numClusters;
+    const void *func;
+    const cudaLaunchConfig_t *launchConfig;
+} cudaOccupancyMaxActiveClusters_v11070_params;
+
 typedef struct cudaMallocManaged_v6000_params_st {
     void **devPtr;
     size_t size;
@@ -1508,9 +1526,7 @@ typedef struct cudaGraphDestroyNode_v10000_params_st {
 typedef struct cudaGraphInstantiate_v12000_params_st {
     cudaGraphExec_t *pGraphExec;
     cudaGraph_t graph;
-    cudaGraphNode_t *pErrorNode;
-    char *pLogBuffer;
-    size_t bufferSize;
+    unsigned long long flags;
 } cudaGraphInstantiate_v12000_params;
 
 typedef struct cudaGraphInstantiateWithFlags_v11040_params_st {
@@ -1983,6 +1999,12 @@ typedef struct cudaLaunchKernel_v7000_params_st {
     cudaStream_t stream;
 } cudaLaunchKernel_v7000_params;
 
+typedef struct cudaLaunchKernelExC_v11060_params_st {
+    const cudaLaunchConfig_t *config;
+    const void *func;
+    void **args;
+} cudaLaunchKernelExC_v11060_params;
+
 typedef struct cudaLaunchCooperativeKernel_v9000_params_st {
     const void *func;
     dim3 gridDim;
