@@ -1,5 +1,5 @@
 /*
- * Copyright 2018-2021 NVIDIA Corporation.  All rights reserved.
+ * Copyright 2018-2023 NVIDIA Corporation.  All rights reserved.
  *
  * NOTICE TO LICENSEE:
  *
@@ -92,65 +92,70 @@ typedef enum {
     /**
      * Invalid domain.
      */
-    SANITIZER_CB_DOMAIN_INVALID     = 0,
+    SANITIZER_CB_DOMAIN_INVALID          = 0,
 
     /**
      * Domain containing callback points for all driver API functions.
      */
-    SANITIZER_CB_DOMAIN_DRIVER_API  = 1,
+    SANITIZER_CB_DOMAIN_DRIVER_API       = 1,
 
     /**
      * Domain containing callback points for all runtime API functions.
      */
-    SANITIZER_CB_DOMAIN_RUNTIME_API = 2,
+    SANITIZER_CB_DOMAIN_RUNTIME_API      = 2,
 
     /**
      * Domain containing callback points for CUDA resource tracking.
      */
-    SANITIZER_CB_DOMAIN_RESOURCE    = 3,
+    SANITIZER_CB_DOMAIN_RESOURCE         = 3,
 
     /**
      * Domain containing callback points for CUDA synchronization.
      */
-    SANITIZER_CB_DOMAIN_SYNCHRONIZE = 4,
+    SANITIZER_CB_DOMAIN_SYNCHRONIZE      = 4,
 
     /**
      * Domain containing callback points for CUDA grid launches.
      */
-    SANITIZER_CB_DOMAIN_LAUNCH      = 5,
+    SANITIZER_CB_DOMAIN_LAUNCH           = 5,
 
     /**
      * Domain containing callback points for CUDA memcpy operations.
      */
-    SANITIZER_CB_DOMAIN_MEMCPY      = 6,
+    SANITIZER_CB_DOMAIN_MEMCPY           = 6,
 
     /**
      * Domain containing callback points for CUDA memset operations.
      */
-    SANITIZER_CB_DOMAIN_MEMSET      = 7,
+    SANITIZER_CB_DOMAIN_MEMSET           = 7,
 
     /**
      * Domain containing callback points for CUDA batch memop operations.
      */
-    SANITIZER_CB_DOMAIN_BATCH_MEMOP = 8,
+    SANITIZER_CB_DOMAIN_BATCH_MEMOP      = 8,
 
     /**
      * Domain containing callback points for CUDA managed memory operations.
      */
-    SANITIZER_CB_DOMAIN_UVM         = 9,
+    SANITIZER_CB_DOMAIN_UVM              = 9,
 
     /**
      * Domain containing callback points for CUDA graphs operations.
      */
-    SANITIZER_CB_DOMAIN_GRAPHS      = 10,
+    SANITIZER_CB_DOMAIN_GRAPHS           = 10,
 
     /**
      * Domain containing callback points for CUDA events.
      */
-    SANITIZER_CB_DOMAIN_EVENTS      = 11,
+    SANITIZER_CB_DOMAIN_EVENTS           = 11,
+
+    /**
+     * Domain containing callback points for CUDA external memory.
+     */
+    SANITIZER_CB_DOMAIN_EXTERNAL_MEMORY  = 12,
 
     SANITIZER_CB_DOMAIN_SIZE,
-    SANITIZER_CB_DOMAIN_FORCE_INT   = 0x7fffffff
+    SANITIZER_CB_DOMAIN_FORCE_INT        = 0x7fffffff
 } Sanitizer_CallbackDomain;
 
 /**
@@ -1441,6 +1446,21 @@ typedef struct {
      */
     uint32_t isDeviceLaunch;
 
+    /**
+     * Boolean value indicating if the graphexec may launch device
+     * graphs. Only valid in the SANITIZER_CBID_GRAPHS_GRAPHEXEC_CREATED
+     * callback with driver version of 535 or newer.
+     */
+    uint32_t containsDeviceGraphLaunches;
+
+    /**
+     * Context where the graphexec can launch device graphs.
+     * NULL if the graphExec doesn't launch device graphs.
+     * Only valid in the SANITIZER_CBID_GRAPHS_GRAPHEXEC_CREATED
+     * callback with driver version of 535 or newer.
+     */
+    CUcontext deviceGraphLaunchesContext;
+
 } Sanitizer_GraphExecData;
 
 /**
@@ -1649,6 +1669,82 @@ typedef struct {
     Sanitizer_StreamHandle hStream;
 } Sanitizer_EventData;
 
+/**
+ * \brief Callback IDs for external memory domain.
+ *
+ * Callback IDs for resource domain
+ * SANITIZER_CB_DOMAIN_EXTERNA_MEMORY. This value is communicated
+ * to the callback function via the \p cbid parameter.
+ * Available with a driver version of 535 or newer.
+ */
+typedef enum {
+    /**
+     * Invalid external memory callback ID.
+     */
+    SANITIZER_CBID_EXTERNAL_MEMORY_INVALID     = 0,
+
+    /**
+     * External memory was imported.
+     */
+    SANITIZER_CBID_EXTERNAL_MEMORY_IMPORT      = 1,
+
+    /**
+     * External memory was mapped.
+     */
+    SANITIZER_CBID_EXTERNAL_MEMORY_MAPPED      = 2,
+
+    /**
+     * External memory was destroyed.
+     */
+    SANITIZER_CBID_EXTERNAL_MEMORY_DESTROYED   = 3,
+
+    SANITIZER_CBID_EXTERNAL_MEMORY_SIZE,
+    SANITIZER_CBID_EXTERNAL_MEMORY_FORCE_INT   = 0x7fffffff
+} Sanitizer_CallbackIdExternalMemory;
+
+/**
+ *
+ * \brief Data passed into an external memory callback function.
+ *
+ * Data passed into an event callback function as the
+ * \p cbdata argument to \ref Sanitizer_CallbackFunc. The
+ * \p cbdata will be this type for \p domain equal tp
+ * SANITIZER_CB_DOMAIN_EXTERNAL_MEMORY.
+ * The callback data is only valid within the invocation of the
+ * callback function that is passed the data. If you need to
+ * retain some data for use outside of the callback, you must
+ * make a copy of it.
+ */
+typedef struct {
+    /**
+     * Context containing the external memory.
+     */
+    CUcontext context;
+
+    /**
+     * Device containing the external memory.
+     */
+    CUdevice device;
+
+    /**
+     * External memory object.
+     */
+    CUexternalMemory extMemory;
+
+    /**
+     * Address of the mapped memory. This field is only valid for
+     * SANITIZER_CBID_EXTERNAL_MEMORY_MAPPED
+     */
+    uint64_t address;
+
+    /**
+     * Size of the memory imported or mapped. This field is only valid
+     * for SANITIZER_CBID_EXTERNAL_MEMORY_IMPORT and
+     * SANITIZER_CBID_EXTERNAL_MEMORY_MAPPED.
+     */
+    uint64_t size;
+} Sanitizer_ExternalMemoryData;
+
 /**
  * \brief Callback ID
  */
