@@ -712,7 +712,24 @@ typedef enum SANITIZER_CBID_DRIVER_API_enum {
    SANITIZER_CBID_DRIVER_API_cuTensorMapEncodeIm2col                                      = 698,
    SANITIZER_CBID_DRIVER_API_cuTensorMapReplaceAddress                                    = 699,
    SANITIZER_CBID_DRIVER_API_cuLibraryGetUnifiedFunction                                  = 700,
-   SANITIZER_CBID_DRIVER_API_SIZE                                                         = 701,
+   SANITIZER_CBID_DRIVER_API_cuCoredumpGetAttribute                                       = 701,
+   SANITIZER_CBID_DRIVER_API_cuCoredumpGetAttributeGlobal                                 = 702,
+   SANITIZER_CBID_DRIVER_API_cuCoredumpSetAttribute                                       = 703,
+   SANITIZER_CBID_DRIVER_API_cuCoredumpSetAttributeGlobal                                 = 704,
+   SANITIZER_CBID_DRIVER_API_cuCtxSetFlags                                                = 705,
+   SANITIZER_CBID_DRIVER_API_cuMulticastCreate                                            = 706,
+   SANITIZER_CBID_DRIVER_API_cuMulticastAddDevice                                         = 707,
+   SANITIZER_CBID_DRIVER_API_cuMulticastBindMem                                           = 708,
+   SANITIZER_CBID_DRIVER_API_cuMulticastBindAddr                                          = 709,
+   SANITIZER_CBID_DRIVER_API_cuMulticastUnbind                                            = 710,
+   SANITIZER_CBID_DRIVER_API_cuMulticastGetGranularity                                    = 711,
+   SANITIZER_CBID_DRIVER_API_cuGraphAddNode                                               = 712,
+   SANITIZER_CBID_DRIVER_API_cuGraphNodeSetParams                                         = 713,
+   SANITIZER_CBID_DRIVER_API_cuGraphExecNodeSetParams                                     = 714,
+   SANITIZER_CBID_DRIVER_API_cuMemAdvise_v2                                               = 715,
+   SANITIZER_CBID_DRIVER_API_cuMemPrefetchAsync_v2                                        = 716,
+   SANITIZER_CBID_DRIVER_API_cuMemPrefetchAsync_v2_ptsz                                   = 717,
+   SANITIZER_CBID_DRIVER_API_SIZE                                                         = 718,
    SANITIZER_CBID_DRIVER_API_FORCE_INT                                                    = 0x7fffffff,
 } SANITIZER_CBID_DRIVER_API;
 
