@@ -1,5 +1,5 @@
 /*
- * Copyright 2018-2021 NVIDIA Corporation.  All rights reserved.
+ * Copyright 2018-2023 NVIDIA Corporation.  All rights reserved.
  *
  * NOTICE TO LICENSEE:
  *
@@ -199,6 +199,11 @@ typedef enum {
      */
     SANITIZER_MEMORY_DEVICE_FLAG_ATOMSYS   = 0x4,
 
+    /**
+     * Specifies that the access is a cache prefetch.
+     */
+    SANITIZER_MEMORY_DEVICE_FLAG_PREFETCH  = 0x8,
+
     SANITIZER_MEMORY_DEVICE_FLAG_FORCE_INT = 0x7fffffff
 } Sanitizer_DeviceMemoryFlags;
 
@@ -214,10 +219,10 @@ typedef enum {
  * 8, and 16.
  * \p flags contains information about the type of access. See
  * Sanitizer_DeviceMemoryFlags to interpret this value.
- * \p newValue is a pointer to the new value being written if the acces is a
+ * \p pNewValue is a pointer to the new value being written if the access is a
  * write. If the access is a read or an atomic, the pointer will be NULL.
  */
-typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackMemoryAccess)(void* userdata, uint64_t pc, void* ptr, uint32_t accessSize, uint32_t flags, const void* newValue);
+typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackMemoryAccess)(void* userdata, uint64_t pc, void* ptr, uint32_t accessSize, uint32_t flags, const void* pNewValue);
 
 /**
  * \brief Flags describing a barrier
@@ -447,7 +452,7 @@ typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackPipelineWait)(void*
  * \p flags contains information about the type of access. See
  * Sanitizer_DeviceMemoryFlags to interpret this value.
  * \p count is the number of matrices accessed.
- * \p newValue is a pointer to the new value being written if the acces is a
+ * \p pNewValue is a pointer to the new value being written if the access is a
  * write. If the access is a read or an atomic, the pointer will be NULL.
  */
 typedef SanitizerPatchResult(SANITIZERAPI* SanitizerCallbackMatrixMemoryAccess)(void* userdata, uint64_t pc, uint32_t address, uint32_t accessSize, uint32_t flags, uint32_t count, const void* pNewValue);
@@ -485,6 +490,121 @@ typedef enum {
  */
 typedef SanitizerPatchResult(SANITIZERAPI* SanitizerCallbackCacheControl)(void* userdata, uint64_t pc, void* address, Sanitizer_CacheControlInstructionKind kind);
 
+/**
+ * \brief Function type for a cluster barrier arrive.
+ *
+ * This can be generated by a cg::this_cluster().sync() (C++ API), or a
+ * barrier.cluster.arrive (PTX API).
+ *
+ * \p userdata is a pointer to user data. See \ref sanitizerPatchModule
+ * \p pc is the program counter of the patched instruction
+ */
+typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackClusterBarrierArrive)(void* userdata, uint64_t pc);
+
+/**
+ * \brief Function type for a cluster barrier wait.
+ *
+ * This can be generated by a cg::this_cluster().sync() (C++ API), or a
+ * barrier.cluster.wait (PTX API).
+ *
+ * \p userdata is a pointer to user data. See \ref sanitizerPatchModule
+ * \p pc is the program counter of the patched instruction
+ */
+typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackClusterBarrierArrive)(void* userdata, uint64_t pc);
+
+/**
+ * \brief Flags describing a warpgroup aligned MMA async.
+ *
+ * Flags describing a warpgroup aligned MMA async. These values are to be or-combined in the
+ * value of \b flags for a SanitizerCallbackWarpgroupMMAAsync callback.
+ */
+typedef enum {
+    /**
+     * Empty flag.
+     */
+    SANITIZER_WARPGROUP_MMA_ASYNC_FLAG_NONE              = 0,
+
+    /**
+     * Specifies that the MMA async delimits a MMA async group of which it is
+     * the last instruction. Please refer to the PTX documentation for wgmma_async.commit_group
+     * for more details. This property is valid even if the warpMask is zero.
+     */
+    SANITIZER_WARPGROUP_MMA_ASYNC_FLAG_COMMIT_GROUP = 0x1,
+
+    SANITIZER_WARPGROUP_MMA_ASYNC_FLAG_FORCE_INT         = 0x7fffffff
+} Sanitizer_WarpgroupMMAAsyncFlags;
+
+/**
+ * \brief Function type for a warpgroup aligned async MMA.
+ *
+ * This can be generated by a wgmma.mma_async in PTX.
+ *
+ * \p userdata is a pointer to user data. See \ref sanitizerPatchModule
+ * \p pc is the program counter of the patched instruction
+ * \p addressMatrixA is the address in shared memory of the matrix A being read. This field is only valid if sizeMatrixA is non-zero and warpMask is full.
+ * \p sizeMatrixA is the size of the matrix A in shared memory. A value of 0 means that the matrix A is read from registers instead.
+ * \p addressMatrixB is the address in shared memory of the matrix B being read. This field is only valid if warpMask is full.
+ * \p sizeMatrixB is the size of the matrix B in shared memory. The value will always be non-zero.
+ * \p flags of type Sanitizer_WarpgroupMMAAsyncFlags provide information about the access. These flags are to be taken into account even if the warpMask is zero.
+ * \p warpMask is a mask of threads that will perform the operation and read the operands. Expected values are either 0x0 or 0xffffffff (full). The value is expected to be the same across the warpgroup.
+ *    Other values can be reported but signal a programming error in the target application.
+ */
+typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackWarpgroupMMAAsync)(void* userdata, uint64_t pc, uint32_t addressMatrixA, uint32_t sizeMatrixA, uint32_t addressMatrixB, uint32_t sizeMatrixB, uint32_t flags, uint32_t warpMask);
+
+/**
+ * \brief Function type for a warpgroup MMA wait group
+ *
+ * This can be generated by a wgmma.wait_group in PTX.
+ *
+ * \p userdata is a pointer to user data. See \ref sanitizerPatchModule.
+ * \p pc is the program counter of the patched instruction.
+ * \p numGroups is the maximum number of group that will be left pending after the operation. A value of zero means that all MMA async of the warpgroup are guaranteed to have completed after the operation.
+ * \p warpMask is a mask of threads for which the expected values are either 0x0 or 0xffffffff (full). The value is expected to be the same across the warpgroup.
+ *    Other values can be reported but signal a programming error in the target application. If the value is valid, the value has no influence on the operation.
+ */
+typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackWarpgroupWaitGroup)(void* userdata, uint64_t pc, uint32_t numGroups, uint32_t warpMask);
+
+/**
+ * \brief Function type for a warpgroup MMA fence
+ *
+ * This can be generated by a wgmma.fence in PTX.
+ *
+ * \p userdata is a pointer to user data. See \ref sanitizerPatchModule.
+ * \p pc is the program counter of the patched instruction.
+ * \p warpMask is a mask of threads that will perform the fence operation. Expected values are either 0x0 or 0xffffffff (full). The value is expected to be the same across the warpgroup.
+ *    Other values can be reported but signal a programming error in the target application.
+ */
+typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackWarpgroupFence)(void* userdata, uint64_t pc, uint32_t warpMask);
+
+/**
+ * \brief Function type for an asynchronous store operation on shared memory
+ *
+ * This can be generated by a st.async PTX instruction
+ *
+ * \p userdata is a pointer to user data. See \ref sanitizerPatchModule.
+ * \p pc is the program counter of the patched instruction.
+ * \p address is the destination address in shared memory.
+ * \p mbarAddress is the address of the mbarrier object.
+ * \p pNewValue is a pointer to the new value being written.
+ * \p accessSize is the size of the access in bytes. Valid values are 4 and 8.
+ *
+ */
+typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackAsyncStore)(void* userdata, uint64_t pc, uint32_t address, uint32_t mbarAddress, void* pNewValue, uint32_t accessSize);
+
+/**
+ * \brief Function type for an asynchronous reduction operation on shared memory
+ *
+ * This can be generated by a red.async PTX instruction
+ *
+ * \p userdata is a pointer to user data. See \ref sanitizerPatchModule.
+ * \p pc is the program counter of the patched instruction.
+ * \p address is the destination address in shared memory.
+ * \p mbarAddress is the address of the mbarrier object.
+ * \p accessSize is the size of the access in bytes. Valid values are 4 and 8.
+ *
+ */
+typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackAsyncReduction)(void* userdata, uint64_t pc, uint32_t address, uint32_t mbarAddress, uint32_t accessSize);
+
 /**
  * \brief Instrumentation.
  *
@@ -612,6 +732,48 @@ typedef enum {
      */
     SANITIZER_INSTRUCTION_CACHE_CONTROL        = 20,
 
+    /**
+     * Cluster barrier arrive instruction. The type of the callback must
+     * be SanitizerCallbackClusterBarrierArrive.
+     */
+    SANITIZER_INSTRUCTION_CLUSTER_BARRIER_ARRIVE = 21,
+
+    /**
+     * Cluster barrier wait instruction. The type of the callback must
+     * be SanitizerCallbackClusterBarrierWait.
+     */
+    SANITIZER_INSTRUCTION_CLUSTER_BARRIER_WAIT = 22,
+
+    /**
+     * Warpgroup aligned async MMA instruction. The type of the callback must
+     * be SanitizerCallbackWarpgroupMMAAsync.
+     */
+    SANITIZER_INSTRUCTION_WARPGROUP_MMA_ASYNC = 23,
+
+    /**
+     * Warpgroup wait MMA group instruction. The type of the callback must
+     * be SanitizerCallbackWarpgroupWaitGroup.
+     */
+    SANITIZER_INSTRUCTION_WARPGROUP_WAIT_GROUP = 24,
+
+    /**
+     * Warpgroup fence instruction. The type of the callback must
+     * be SanitizerCallbackWarpgroupFence.
+     */
+    SANITIZER_INSTRUCTION_WARPGROUP_FENCE = 25,
+
+    /**
+     * Asynchronous store instruction. The type of the callback must
+     * be SanitizerCallbackAsyncStore.
+     */
+    SANITIZER_INSTRUCTION_ASYNC_STORE          = 26,
+
+    /**
+     * Asynchronous reduction instruction. The type of the callback must
+     * be SanitizerCallbackAsyncReduction.
+     */
+    SANITIZER_INSTRUCTION_ASYNC_REDUCTION      = 27,
+
     SANITIZER_INSTRUCTION_FORCE_INT            = 0x7fffffff
 } Sanitizer_InstructionId;
 
@@ -702,6 +864,26 @@ SanitizerResult SANITIZERAPI sanitizerSetLaunchCallbackData(Sanitizer_LaunchHand
                                                             Sanitizer_StreamHandle stream,
                                                             const void* userdata);
 
+/**
+ * \brief Specifies the user data pointer accessible from callbacks in the
+ * device-launched graphs launched by the specified host-launched graphExec.
+ *
+ * Mark all subsequent launch of \p graphExec to make available \p userdata
+ * in device callbacks from device-launched graphs. \p userdata will not
+ * be set in the callback userdata parameter but must be accessed through
+ * another mean instead. Please refer to the Sanitizer API reference manual.
+ * This function is only available if the driver version is 535 or newer.
+ *
+ * \param graphExec CUDA graphExec that will launch CUDA graphs from the device.
+ * \param stream CUDA stream associated with the stream launch.
+ * \param userdata Device memory buffer.
+ *
+ * \retval SANITIZER_SUCCESS on success
+ */
+SanitizerResult SANITIZERAPI sanitizerSetDeviceGraphData(CUgraphExec graphExec,
+                                                         Sanitizer_StreamHandle stream,
+                                                         const void* userdata);
+
 /**
  *
  * \brief Remove existing instrumentation of a module
