@@ -441,7 +441,10 @@ typedef enum SANITIZER_CBID_RUNTIME_API_enum {
     SANITIZER_CBID_RUNTIME_API_cudaStreamGetId_ptsz                                        = 442,
     SANITIZER_CBID_RUNTIME_API_cudaGraphInstantiate                                        = 443,
     SANITIZER_CBID_RUNTIME_API_cudaInitDevice                                              = 444,
-    SANITIZER_CBID_RUNTIME_API_SIZE                                                        = 445,
+    SANITIZER_CBID_RUNTIME_API_cudaGraphAddNode                                            = 445,
+    SANITIZER_CBID_RUNTIME_API_cudaGraphNodeSetParams                                      = 446,
+    SANITIZER_CBID_RUNTIME_API_cudaGraphExecNodeSetParams                                  = 447,
+    SANITIZER_CBID_RUNTIME_API_SIZE                                                        = 448,
     SANITIZER_CBID_RUNTIME_API_FORCE_INT                                                   = 0x7fffffff
 } SANITIZER_CBID_RUNTIME_API;
 
