@@ -87,6 +87,18 @@ cublasStatus_t CUBLASWINAPI cublasLtGetProperty(libraryPropertyType type, int* v
 cublasStatus_t CUBLASWINAPI cublasLtHeuristicsCacheGetCapacity(size_t* capacity);
 cublasStatus_t CUBLASWINAPI cublasLtHeuristicsCacheSetCapacity(size_t capacity);
 
+/** Restricts usage of CPU instructions (ISA) specified by the flags in the mask.
+ *
+ * Flags can be combined with bitwise OR(|) operator. Supported flags:
+ * - 0x1 -- x86-64 AVX512 ISA
+ *
+ * Default mask: 0 (any applicable ISA is allowed).
+ *
+ * The function returns the previous value of the mask.
+ * The function takes precedence over the environment variable CUBLASLT_DISABLE_CPU_INSTRUCTIONS_MASK.
+ */
+unsigned CUBLASWINAPI cublasLtDisableCpuInstructionsSetMask(unsigned mask);
+
 /** Semi-opaque descriptor for matrix memory layout
  */
 typedef struct {
@@ -175,6 +187,8 @@ typedef enum {
   CUBLASLT_MATMUL_TILE_192x128 = 31,
   CUBLASLT_MATMUL_TILE_128x192 = 32,
   CUBLASLT_MATMUL_TILE_128x96 = 33,
+  CUBLASLT_MATMUL_TILE_32x256 = 34,
+  CUBLASLT_MATMUL_TILE_256x32 = 35,
   CUBLASLT_MATMUL_TILE_END
 } cublasLtMatmulTile_t;
 
