@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2017 NVIDIA CORPORATION. All rights reserved.
+ * Copyright (c) 2014-2023 NVIDIA CORPORATION. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -45,8 +45,8 @@
 #define ERRMSG_LEN			256
 
 static __THREAD char lastErrMsg[ERRMSG_LEN];
-static int cuCoreAddMapEntry(MapEntry **, void *, int, const char *, ...)
- _PRINTF_ARGS(4,5);
+static int cuCoreAddMapEntry(MapEntry **, void *, size_t, bool, const char *, ...)
+ _PRINTF_ARGS(5, 6);
 
 void cuCoreSetErrorMsg(const char *fmt, ...)
 {
@@ -221,8 +221,7 @@ static int cuCoreReadMemorySection(Elf *e, Elf_Scn *scn,
 
 static int cuCoreGenericReadTable(Elf *e, Elf_Scn *scn,
 				  size_t *count, void **table,
-				  size_t entry_size,
-                                  size_t *shentsize,
+				  size_t *shentsize,
 				  size_t *parent, size_t *offset)
 {
   Elf_Data data;
@@ -238,8 +237,7 @@ static int cuCoreGenericReadTable(Elf *e, Elf_Scn *scn,
     *parent = readUint32(&shdr->sh_link);
   if (offset)
     *offset = readUint32(&shdr->sh_info);
-  if (shentsize)
-    *shentsize = readUint64(&shdr->sh_entsize);
+  *shentsize = readUint64(&shdr->sh_entsize);
 
   *count = data.d_size / readUint64(&shdr->sh_entsize);
   *table = data.d_buf;
@@ -247,7 +245,7 @@ static int cuCoreGenericReadTable(Elf *e, Elf_Scn *scn,
   return 0;
 }
 
-static int cuCoreAddMapEntry(MapEntry **map, void *entryPtr, int needsFree,
+static int cuCoreAddMapEntry(MapEntry **map, void *entryPtr, size_t size, bool needsFree,
 			     const char *fmt, ...)
 {
 	MapEntry *mapEntry;
@@ -260,7 +258,8 @@ static int cuCoreAddMapEntry(MapEntry **map, void *entryPtr, int needsFree,
 
 	vsnprintf(mapEntry->ident, MAPIDENT_LEN, fmt, args);
 	mapEntry->entryPtr = entryPtr;
-        mapEntry->needsFree = needsFree;
+	mapEntry->size = size;
+	mapEntry->needsFree = needsFree;
 
 	DPRINTF(80, "Mapped '%s' to %p needsFree %d\n",
                 mapEntry->ident, entryPtr, needsFree);
@@ -270,7 +269,7 @@ static int cuCoreAddMapEntry(MapEntry **map, void *entryPtr, int needsFree,
 	return 0;
 }
 
-void *cuCoreGetMapEntry(MapEntry **map, const char *fmt, ...)
+void *cuCoreGetMapEntry(MapEntry **map, size_t* size, const char *fmt, ...)
 {
 	MapEntry *mapEntry;
 	va_list args;
@@ -285,13 +284,17 @@ void *cuCoreGetMapEntry(MapEntry **map, const char *fmt, ...)
 	DPRINTF(90, "Got map entry '%s' %p\n",
 		mapEntry->ident, mapEntry->entryPtr);
 
+	if (size)
+	  {
+		*size = mapEntry->size;
+	  }
+
 	return mapEntry->entryPtr;
 }
 
 static int cuCoreReadDeviceTable(CudaCore *cc, Elf_Scn *scn)
 {
   uint8_t *dt;
-  size_t dteCoreSz;
   size_t dte_count;
   CudbgDeviceTableEntry *dte;
   size_t dteSz;
@@ -300,28 +303,21 @@ static int cuCoreReadDeviceTable(CudaCore *cc, Elf_Scn *scn)
   if (cuCoreGenericReadTable(cc->e, scn,
                              &dte_count,
                              (void **)&dt,
-                             sizeof(CudbgDeviceTableEntry),
-                             &dteCoreSz, NULL, NULL) != 0)
+                             &dteSz, NULL, NULL) != 0)
     return -1;
 
   cc->numDevices = dte_count;
 
-  dteSz = (dteCoreSz <= sizeof(CudbgDeviceTableEntry)) ? dteCoreSz : sizeof(CudbgDeviceTableEntry);
-
-  for (i = 0; i < dte_count; ++i) {
-    dte = (CudbgDeviceTableEntry *)calloc(1, sizeof(*dte));
-    VERIFY(dte != NULL, -1, "Could not allocate memory");
-
-    memcpy(dte, dt, dteSz);
-    dt += dteCoreSz;
+  for (i = 0; i < dte_count; ++i, dt += dteSz) {
+    dte = (CudbgDeviceTableEntry *)dt;
 
     /* delete the dte if this entry is deleted */
-    if (cuCoreAddMapEntry(&cc->tableEntriesMap, dte, 1,
+    if (cuCoreAddMapEntry(&cc->tableEntriesMap, dte, dteSz, false,
                           "dev%u",
                           dte->devId))
       return -1;
 
-    if (cuCoreAddMapEntry(&cc->tableEntriesMap, dte, 0,
+    if (cuCoreAddMapEntry(&cc->tableEntriesMap, dte, dteSz, false,
                           "devtbl_offset%llu", (unsigned long long)i))
       return -1;
   }
@@ -332,40 +328,42 @@ static int cuCoreReadDeviceTable(CudaCore *cc, Elf_Scn *scn)
 static int cuCoreReadGridTable(CudaCore *cc, Elf_Scn *scn)
 {
 	size_t gte_count;
-	CudbgGridTableEntry *gt, *gte;
+	uint8_t *gt;
+	CudbgGridTableEntry *gte;
 	CudbgDeviceTableEntry *dte;
+	size_t dteSz, gteSz;
 	size_t parent, offset;
 	size_t i;
 
 	if (cuCoreGenericReadTable(cc->e, scn,
 				   &gte_count,
 				   (void **)&gt,
-				   sizeof(CudbgGridTableEntry),
-				   NULL, &parent, &offset) != 0)
+				   &gteSz, &parent, &offset) != 0)
 		return -1;
 
 	dte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				&dteSz,
 				"devtbl_offset%llu",
 				(unsigned long long)offset);
 	VERIFY(dte != NULL, -1, "Could not find Device table entry");
 
-	for (i = 0; i < gte_count; ++i) {
-		gte = &gt[i];
+	for (i = 0; i < gte_count; ++i, gt += gteSz) {
+		gte = (CudbgGridTableEntry *)gt;
 
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, gte, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, gte, gteSz, false,
 				      "grid%llu_dev%u",
 				      (unsigned long long)gte->gridId64,
 				      dte->devId))
 			return -1;
 
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, gte, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, gte, gteSz, false,
 				      "grid_section%llu_offset%llu",
 					  (unsigned long long)elfGetSectionIndex(cc->e, scn),
 					  (unsigned long long)i))
 			return -1;
 
 		/* Index dte by grid section and offset */
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, dte, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, dte, dteSz, false,
 				      "grid_section%llu_offset%llu_dev",
 					  (unsigned long long)elfGetSectionIndex(cc->e, scn),
 					  (unsigned long long)i))
@@ -378,39 +376,40 @@ static int cuCoreReadGridTable(CudaCore *cc, Elf_Scn *scn)
 static int cuCoreReadSmTable(CudaCore *cc, Elf_Scn *scn)
 {
 	size_t ste_count;
-	CudbgSmTableEntry *st, *ste;
+	uint8_t *st;
+	CudbgSmTableEntry *ste;
 	CudbgDeviceTableEntry *dte;
+	size_t dteSz, steSz;
 	size_t parent, offset;
 	size_t i;
 
 	if (cuCoreGenericReadTable(cc->e, scn,
 				   &ste_count,
 				   (void **)&st,
-				   sizeof(CudbgSmTableEntry),
-				   NULL, &parent, &offset) != 0)
+				   &steSz, &parent, &offset) != 0)
 		return -1;
 
 	dte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				&dteSz,
 				"devtbl_offset%llu",
 				(unsigned long long)offset);
 	VERIFY(dte != NULL, -1, "Could not find Device table entry");
 
-	for (i = 0; i < ste_count; ++i) {
-		ste = &st[i];
-
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, ste, 0,
+	for (i = 0; i < ste_count; ++i, st += steSz) {
+		ste = (CudbgSmTableEntry *)st;
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, ste, steSz, false,
 				      "sm%u_dev%u",
 				      ste->smId,
 				      dte->devId))
 			return -1;
 
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, ste, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, ste, steSz, false,
 				      "smtbl_section%llu_offset%llu",
 					  (unsigned long long)elfGetSectionIndex(cc->e, scn), (unsigned long long)i))
 			return -1;
 
 		/* Link SM table entry to device */
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, dte, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, dte, dteSz, false,
 				      "smtbl_section%llu_offset%llu_dev",
 					  (unsigned long long)elfGetSectionIndex(cc->e, scn), (unsigned long long)i))
 			return -1;
@@ -422,50 +421,50 @@ static int cuCoreReadSmTable(CudaCore *cc, Elf_Scn *scn)
 static int cuCoreReadCTATable(CudaCore *cc, Elf_Scn *scn)
 {
 	size_t ctate_count;
-	CudbgCTATableEntry *ctat, *ctate;
+	uint8_t *ctate;
 	CudbgSmTableEntry *ste;
 	CudbgDeviceTableEntry *dte;
+	size_t dteSz, steSz, ctateSz;
 	size_t parent, offset;
 	size_t i;
 
 	if (cuCoreGenericReadTable(cc->e, scn,
 				   &ctate_count,
-				   (void **)&ctat,
-				   sizeof(CudbgCTATableEntry),
-				   NULL, &parent, &offset) != 0)
+				   (void **)&ctate,
+				   &ctateSz, &parent, &offset) != 0)
 		return -1;
 
 	ste = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				&steSz,
 				"smtbl_section%llu_offset%llu",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(ste != NULL, -1, "Could not find SM table entry");
 
 	dte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				&dteSz,
 				"smtbl_section%llu_offset%llu_dev",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(dte != NULL, -1, "Could not find Device table entry by SM");
 
-	for (i = 0; i < ctate_count; ++i) {
-		ctate = &ctat[i];
-
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, ctate, 0,
+	for (i = 0; i < ctate_count; ++i, ctate += ctateSz) {
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, ctate, ctateSz, false,
 				      "cta%u_sm%u_dev%u",
 				      (uint32_t) i,
 				      ste->smId,
 				      dte->devId))
 			return -1;
 
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, ctate, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, ctate, ctateSz, false,
 				      "ctatbl_section%llu_offset%llu",
 					  (unsigned long long)elfGetSectionIndex(cc->e, scn), (unsigned long long)i))
 			return -1;
 
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, ste, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, ste, steSz, false,
 				      "ctatbl_section%llu_offset%llu_sm",
 					  (unsigned long long)elfGetSectionIndex(cc->e, scn), (unsigned long long)i))
 			return -1;
 
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, dte, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, dte, dteSz, false,
 				      "ctatbl_section%llu_offset%llu_dev",
 					  (unsigned long long)elfGetSectionIndex(cc->e, scn), (unsigned long long)i))
 			return -1;
@@ -477,63 +476,67 @@ static int cuCoreReadCTATable(CudaCore *cc, Elf_Scn *scn)
 static int cuCoreReadWarpTable(CudaCore *cc, Elf_Scn *scn)
 {
 	size_t wte_count;
-	CudbgWarpTableEntry *wt, *wte;
+	uint8_t *wt;
+	CudbgWarpTableEntry *wte;
 	CudbgCTATableEntry *ctate;
 	CudbgSmTableEntry *ste;
 	CudbgDeviceTableEntry *dte;
+	size_t dteSz, steSz, ctateSz, wteSz;
 	size_t parent, offset;
 	size_t i;
 
 	if (cuCoreGenericReadTable(cc->e, scn,
 				   &wte_count,
 				   (void **)&wt,
-				   sizeof(CudbgWarpTableEntry),
-				   NULL, &parent, &offset) != 0)
+				   &wteSz, &parent, &offset) != 0)
 		return -1;
 
 	ctate = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				  &ctateSz,
 				  "ctatbl_section%llu_offset%llu",
 				  (unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(ctate != NULL, -1, "Could not find CTA table entry");
 
 	ste = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				&steSz,
 				"ctatbl_section%llu_offset%llu_sm",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(ste != NULL, -1, "Could not find SM table entry by CTA");
 
 	dte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				&dteSz,
 				"ctatbl_section%llu_offset%llu_dev",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(dte != NULL, -1, "Could not find Device table entry by CTA");
 
-	for (i = 0; i < wte_count; ++i) {
-		wte = &wt[i];
+	for (i = 0; i < wte_count; ++i, wt += wteSz) {
+		wte = (CudbgWarpTableEntry *)wt;
 
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, wte, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, wte, wteSz, false,
 				      "wp%u_sm%u_dev%u",
 				      wte->warpId,
 				      ste->smId,
 				      dte->devId))
 			return -1;
 
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, ctate, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, ctate, ctateSz, false,
 				      "wp%u_sm%u_dev%u_cta",
 				      wte->warpId,
 				      ste->smId,
 				      dte->devId))
 			return -1;
 
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, wte, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, wte, wteSz, false,
 				      "wptbl_section%llu_offset%llu",
                                       (unsigned long long)elfGetSectionIndex(cc->e, scn), (unsigned long long)i))
 			return -1;
 
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, ste, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, ste, steSz, false,
 				      "wptbl_section%llu_offset%llu_sm",
 					  (unsigned long long)elfGetSectionIndex(cc->e, scn), (unsigned long long)i))
 			return -1;
 
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, dte, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, dte, dteSz, false,
 				      "wptbl_section%llu_offset%llu_dev",
 					  (unsigned long long)elfGetSectionIndex(cc->e, scn), (unsigned long long)i))
 			return -1;
@@ -545,57 +548,61 @@ static int cuCoreReadWarpTable(CudaCore *cc, Elf_Scn *scn)
 static int cuCoreReadThreadTable(CudaCore *cc, Elf_Scn *scn)
 {
 	size_t tt_count;
-	CudbgThreadTableEntry *tt, *tte;
+	uint8_t *tt;
+	CudbgThreadTableEntry *tte;
 	CudbgWarpTableEntry *wte;
 	CudbgSmTableEntry *ste;
 	CudbgDeviceTableEntry *dte;
+	size_t dteSz, steSz, wteSz, tteSz;
 	size_t parent, offset;
 	size_t i;
 
 	cuCoreGenericReadTable(cc->e, scn, &tt_count, (void **)&tt,
-			       sizeof(CudbgThreadTableEntry),
-			       NULL, &parent, &offset);
+			       &tteSz, &parent, &offset);
 
 	wte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				&wteSz,
 				"wptbl_section%llu_offset%llu",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(wte != NULL, -1, "Could not find Warp table entry");
 
 	ste = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				&steSz,
 				"wptbl_section%llu_offset%llu_sm",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(ste != NULL, -1, "Could not find SM table entry by Warp");
 
 	dte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				&dteSz,
 				"wptbl_section%llu_offset%llu_dev",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(dte != NULL, -1, "Could not find Device table entry by Warp");
 
-	for (i = 0; i < tt_count; ++i) {
-		tte = &tt[i];
+	for (i = 0; i < tt_count; ++i, tt += tteSz) {
+		tte = (CudbgThreadTableEntry *)tt;
 
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, tte, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, tte, tteSz, false,
 				      "ln%u_wp%u_sm%u_dev%u",
 				      tte->ln, wte->warpId,
 				      ste->smId, dte->devId))
 			return -1;
 
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, tte, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, tte, tteSz, false,
 				      "lntbl_section%llu_offset%llu",
 					  (unsigned long long)elfGetSectionIndex(cc->e, scn), (unsigned long long)i))
 			return -1;
 
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, wte, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, wte, wteSz, false,
 				      "lntbl_section%llu_offset%llu_wp",
 					  (unsigned long long)elfGetSectionIndex(cc->e, scn), (unsigned long long)i))
 			return -1;
 
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, ste, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, ste, steSz, false,
 				      "lntbl_section%llu_offset%llu_sm",
 					  (unsigned long long)elfGetSectionIndex(cc->e, scn), (unsigned long long)i))
 			return -1;
 
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, dte, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, dte, dteSz, false,
 				      "lntbl_section%llu_offset%llu_dev",
 					  (unsigned long long)elfGetSectionIndex(cc->e, scn), (unsigned long long)i))
 			return -1;
@@ -607,42 +614,47 @@ static int cuCoreReadThreadTable(CudaCore *cc, Elf_Scn *scn)
 static int cuCoreReadBacktraceTable(CudaCore *cc, Elf_Scn *scn)
 {
 	size_t bt_count;
-	CudbgBacktraceTableEntry *bt, *bte;
+	uint8_t *bt;
+	CudbgBacktraceTableEntry *bte;
 	CudbgThreadTableEntry *tte;
 	CudbgWarpTableEntry *wte;
 	CudbgSmTableEntry *ste;
 	CudbgDeviceTableEntry *dte;
+	size_t dteSz, steSz, wteSz, tteSz, bteSz;
 	size_t parent, offset;
 	size_t i;
 
 	cuCoreGenericReadTable(cc->e, scn, &bt_count, (void **)&bt,
-			       sizeof(CudbgBacktraceTableEntry),
-			       NULL, &parent, &offset);
+			       &bteSz, &parent, &offset);
 
 	tte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				&tteSz,
 				"lntbl_section%llu_offset%llu",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(tte != NULL, -1, "Could not find Thread table entry");
 
 	wte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				&wteSz,
 				"lntbl_section%llu_offset%llu_wp",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(wte != NULL, -1, "Could not find Warp table entry by Lane");
 
 	ste = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				&steSz,
 				"lntbl_section%llu_offset%llu_sm",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(ste != NULL, -1, "Could not find SM table entry by Lane");
 
 	dte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				&dteSz,
 				"lntbl_section%llu_offset%llu_dev",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(dte != NULL, -1, "Could not find Device table entry by Lane");
 
-	for (i = 0; i < bt_count; ++i) {
-		bte = &bt[i];
+	for (i = 0; i < bt_count; ++i, bt += bteSz) {
+		bte = (CudbgBacktraceTableEntry *)bt;
 
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, bte, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, bte, bteSz, false,
 				      "bt%u_ln%u_wp%u_sm%u_dev%u",
 				      bte->level, tte->ln,
 				      wte->warpId, ste->smId,
@@ -656,8 +668,10 @@ static int cuCoreReadBacktraceTable(CudaCore *cc, Elf_Scn *scn)
 static int cuCoreReadContextTable(CudaCore *cc, Elf_Scn *scn)
 {
 	size_t cte_count;
-	CudbgContextTableEntry *ct, *cte;
+	uint8_t *ct;
+	CudbgContextTableEntry *cte;
 	CudbgDeviceTableEntry *dte;
+	size_t dteSz, cteSz;
 	size_t parent, offset;
 	size_t i;
 	CUDBGEvent event;
@@ -665,25 +679,25 @@ static int cuCoreReadContextTable(CudaCore *cc, Elf_Scn *scn)
 	if (cuCoreGenericReadTable(cc->e, scn,
 				   &cte_count,
 				   (void **)&ct,
-				   sizeof(CudbgContextTableEntry),
-				   NULL, &parent, &offset) != 0)
+				   &cteSz, &parent, &offset) != 0)
 		return -1;
 
-	for (i = 0; i < cte_count; ++i) {
-		cte = &ct[i];
+	for (i = 0; i < cte_count; ++i, ct += cteSz) {
+		cte = (CudbgContextTableEntry *)ct;
 
 		dte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+					&dteSz,
 					"devtbl_offset%u",
 					cte->deviceIdx);
 		VERIFY(dte != NULL, -1, "Could not find Device table entry");
 
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, cte, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, cte, cteSz, false,
 				      "ctx%llu_dev%u",
 				      (unsigned long long)cte->contextId,
 				      dte->devId))
 			return -1;
 
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, cte, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, cte, cteSz, false,
 				      "ctxtbl_section%llu_offset%llu",
 					  (unsigned long long)elfGetSectionIndex(cc->e, scn), (unsigned long long)i))
 			return -1;
@@ -710,29 +724,28 @@ static int cuCoreReadContextTable(CudaCore *cc, Elf_Scn *scn)
 static int cuCoreReadModuleTable(CudaCore *cc, Elf_Scn *scn)
 {
 	size_t mt_count;
-	CudbgModuleTableEntry *mt, *mte;
+	uint8_t *mte;
 	CudbgContextTableEntry *cte;
+	size_t cteSz, mteSz;
 	size_t parent, offset;
 	size_t i;
 
-	cuCoreGenericReadTable(cc->e, scn, &mt_count, (void **)&mt,
-			       sizeof(CudbgModuleTableEntry),
-			       NULL, &parent, &offset);
+	cuCoreGenericReadTable(cc->e, scn, &mt_count, (void **)&mte,
+			       &mteSz, &parent, &offset);
 
 	cte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				&cteSz,
 				"ctxtbl_section%llu_offset%llu",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(cte != NULL, -1, "Could not find Context table entry");
 
-	for (i = 0; i < mt_count; ++i) {
-		mte = &mt[i];
-
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, mte, 0,
+	for (i = 0; i < mt_count; ++i, mte += mteSz) {
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, mte, mteSz, false,
 				      "modtbl_section%llu_offset%llu",
 					  (unsigned long long)elfGetSectionIndex(cc->e, scn), (unsigned long long)i))
 			return -1;
 
-		if (cuCoreAddMapEntry(&cc->tableEntriesMap, cte, 0,
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, cte, cteSz, false,
 				      "modtbl_section%llu_offset%llu_ctx",
 					  (unsigned long long)elfGetSectionIndex(cc->e, scn), (unsigned long long)i))
 			return -1;
@@ -754,16 +767,18 @@ static int cuCoreReadSharedMemorySection(CudaCore *cc, Elf_Scn *scn)
 	offset = shdr->sh_info;
 
 	ste = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				NULL,
 				"ctatbl_section%llu_offset%llu_sm",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(ste != NULL, -1, "Could not find SM table entry");
 
 	dte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				NULL,
 				"ctatbl_section%llu_offset%llu_dev",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(dte != NULL, -1, "Could not find Device table entry");
 
-	if (cuCoreAddMapEntry(&cc->tableEntriesMap, scn, 0,
+	if (cuCoreAddMapEntry(&cc->tableEntriesMap, scn, shdr->sh_size, false,
 			      "cta%llu_sm%u_dev%u_shared",
 			      (unsigned long long)offset,
 			      ste->smId,
@@ -789,26 +804,30 @@ static int cuCoreReadLocalMemorySection(CudaCore *cc, Elf_Scn *scn)
 	offset = shdr->sh_info;
 
 	tte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				NULL,
 				"lntbl_section%llu_offset%llu",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(tte != NULL, -1, "Could not find Thread table entry");
 
 	wte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				NULL,
 				"lntbl_section%llu_offset%llu_wp",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(wte != NULL, -1, "Could not find Warp table entry by Lane");
 
 	ste = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				NULL,
 				"lntbl_section%llu_offset%llu_sm",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(ste != NULL, -1, "Could not find SM table entry by Lane");
 
 	dte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				NULL,
 				"lntbl_section%llu_offset%llu_dev",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(dte != NULL, -1, "Could not find Device table entry by Lane");
 
-	if (cuCoreAddMapEntry(&cc->tableEntriesMap, scn, 0,
+	if (cuCoreAddMapEntry(&cc->tableEntriesMap, scn, shdr->sh_size, false,
 			      "ln%u_wp%u_sm%u_dev%u_local",
 			      tte->ln,
 			      wte->warpId,
@@ -833,16 +852,18 @@ static int cuCoreReadParamMemorySection(CudaCore *cc, Elf_Scn *scn)
 	offset = shdr->sh_info;
 
 	gte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				NULL,
 				"grid_section%llu_offset%llu",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(gte != NULL, -1, "Could not find Grid table entry");
 
 	dte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				NULL,
 				"grid_section%llu_offset%llu_dev",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(dte != NULL, -1, "Could not find Device table entry by Grid");
 
-	if (cuCoreAddMapEntry(&cc->tableEntriesMap, scn, 0,
+	if (cuCoreAddMapEntry(&cc->tableEntriesMap, scn, shdr->sh_size, false,
 			      "grid%llu_dev%u_param",
 			      (unsigned long long)gte->gridId64,
 			      dte->devId))
@@ -890,12 +911,13 @@ static int cuCoreReadELFImage(CudaCore *cc, Elf_Scn *scn, bool reloc)
 		return -1;
 
 	mte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				NULL,
 				"modtbl_section%u_offset%u",
 				hdr->sh_link, hdr->sh_info);
 	VERIFY(mte != NULL, -1, "Could not find Module table entry");
 
 	/* Hash the ELFs SCN by module handle */
-	if (cuCoreAddMapEntry(&cc->tableEntriesMap, scn, 0,
+	if (cuCoreAddMapEntry(&cc->tableEntriesMap, scn, hdr->sh_size, false,
 			      "%celf_handle%llx",
 			      reloc ? 'r' : 'u',
 			      (unsigned long long)readUint64(&mte->moduleHandle)))
@@ -905,11 +927,13 @@ static int cuCoreReadELFImage(CudaCore *cc, Elf_Scn *scn, bool reloc)
 		return 0;
 
 	cte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				NULL,
 				"modtbl_section%u_offset%u_ctx",
 				hdr->sh_link, hdr->sh_info);
 	VERIFY(cte != NULL, -1, "Could not find Context table entry");
 
 	dte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				NULL,
 				"devtbl_offset%u",
 				cte->deviceIdx);
 	VERIFY(dte != NULL, -1, "Could not find Device table entry");
@@ -956,22 +980,25 @@ static int cuCoreReadWarpInfo(const char *key, CudaCore *cc, Elf_Scn *scn)
 	offset = shdr->sh_info;
 
 	wte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				NULL,
 				"wptbl_section%llu_offset%llu",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(wte != NULL, -1, "Could not find Warp table entry by Lane");
 
 	ste = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				NULL,
 				"wptbl_section%llu_offset%llu_sm",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(ste != NULL, -1, "Could not find SM table entry by Lane");
 
 	dte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				NULL,
 				"wptbl_section%llu_offset%llu_dev",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(dte != NULL, -1, "Could not find Device table entry by Lane");
 
 	/* Hash by dev, sm, wp and ln indexes */
-	if (cuCoreAddMapEntry(&cc->tableEntriesMap, scn, 0,
+	if (cuCoreAddMapEntry(&cc->tableEntriesMap, scn, shdr->sh_size, false,
 			      "%s_dev%u_sm%u_wp%u", key,
 			      dte->devId, ste->smId, wte->warpId))
 		return -1;
@@ -995,27 +1022,31 @@ static int cuCoreReadThreadInfo(const char *key, CudaCore *cc, Elf_Scn *scn)
 	offset = shdr->sh_info;
 
 	tte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				NULL,
 				"lntbl_section%llu_offset%llu",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(tte != NULL, -1, "Could not find Thread table entry");
 
 	wte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				NULL,
 				"lntbl_section%llu_offset%llu_wp",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(wte != NULL, -1, "Could not find Warp table entry by Lane");
 
 	ste = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				NULL,
 				"lntbl_section%llu_offset%llu_sm",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(ste != NULL, -1, "Could not find SM table entry by Lane");
 
 	dte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				NULL,
 				"lntbl_section%llu_offset%llu_dev",
 				(unsigned long long)parent, (unsigned long long)offset);
 	VERIFY(dte != NULL, -1, "Could not find Device table entry by Lane");
 
 	/* Hash by dev, sm, wp and ln indexes */
-	if (cuCoreAddMapEntry(&cc->tableEntriesMap, scn, 0,
+	if (cuCoreAddMapEntry(&cc->tableEntriesMap, scn, shdr->sh_size, false,
 			      "%s_dev%u_sm%u_wp%u_ln%u", key,
 			      dte->devId, ste->smId, wte->warpId, tte->ln))
 		return -1;
