@@ -805,6 +805,59 @@ typedef struct
     int nReserved0;
 } NppStreamContext;
 
+/** 
+ * NPP Batch Geometry Structure Definitions. 
+ */
+
+typedef struct
+{
+    const void * pSrc;  /* device memory pointer */
+    int nSrcStep;
+    void * pDst;        /* device memory pointer */
+    int nDstStep;
+} NppiResizeBatchCXR;
+
+/**
+ * Data structure for variable ROI image resizing.
+ * 
+ */
+typedef struct
+{
+    NppiRect oSrcRectROI;    
+    NppiRect oDstRectROI;
+} NppiResizeBatchROI_Advanced; 
+ 
+typedef struct
+{
+    const void * pSrc;  /* device memory pointer, ignored for in place versions */
+    int nSrcStep;
+    void * pDst;        /* device memory pointer */
+    int nDstStep;
+} NppiMirrorBatchCXR;
+
+typedef struct
+{
+    const void * pSrc;  /* device memory pointer */
+    int nSrcStep;
+    void * pDst;        /* device memory pointer */
+    int nDstStep;
+    Npp64f * pCoeffs;   /* device memory pointer to the tranformation matrix with double precision floating-point coefficient values to be used for this image */
+    Npp64f aTransformedCoeffs[2][3]; /* FOR INTERNAL USE, DO NOT INITIALIZE  */
+} NppiWarpAffineBatchCXR;
+
+typedef struct
+{
+    const void * pSrc;  /* device memory pointer */
+    int nSrcStep;
+    void * pDst;        /* device memory pointer */
+    int nDstStep;
+    Npp64f * pCoeffs;   /* device memory pointer to the tranformation matrix with double precision floating-point coefficient values to be used for this image */
+    Npp64f aTransformedCoeffs[3][3]; /* FOR INTERNAL USE, DO NOT INITIALIZE  */
+} NppiWarpPerspectiveBatchCXR;
+
+
+
+
 #ifdef __cplusplus
 #ifndef NPP_PLUS
 } /* extern "C" */
