@@ -22,9 +22,11 @@
 # OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-import NvRules
 import math
 
+import NvRules
+
+
 def get_identifier():
     return "LaunchConfiguration"
 
@@ -54,7 +56,7 @@ def apply(handle):
     doc_msg = " See the @url:Hardware Model:https://docs.nvidia.com/nsight-compute/ProfilingGuide/index.html#metrics-hw-model@ description for more details on launch configurations."
 
     if block_size % 32 != 0:
-        msg_id = fe.message(NvRules.IFrontend.MsgType_MSG_WARNING,\
+        msg_id = fe.message(NvRules.IFrontend.MsgType_MSG_OPTIMIZATION,\
             "Threads are executed in groups of 32 threads called warps. This kernel launch is configured to execute {:d} threads per block."\
             " Consequently, some threads in a warp are masked off and those hardware resources are unused."\
             " Try changing the number of threads per block to be a multiple of 32 threads. Between 128 and 256 threads per block is a good initial range for experimentation."\
@@ -62,26 +64,30 @@ def apply(handle):
             " This is particularly beneficial to kernels that frequently call __syncthreads()."\
             "{}".format(int(block_size), doc_msg), \
             "Block Size")
-        fe.focus_metric(msg_id, "launch__block_size", block_size, NvRules.IFrontend.Severity_SEVERITY_LOW, "{} % 32 != 0".format(block_size))
+        improvement_percent = ((block_size % 32) / 32) / (block_size / 32 + 1) * 100
+        fe.speedup(msg_id, NvRules.IFrontend.SpeedupType_LOCAL, improvement_percent)
+        fe.focus_metric(msg_id, "launch__block_size", block_size, NvRules.IFrontend.Severity_SEVERITY_LOW, "Arrange the number of threads per block to be a multiple of 32")
 
     if grid_size < num_sms:
-        msg_id = fe.message(NvRules.IFrontend.MsgType_MSG_WARNING,\
+        msg_id = fe.message(NvRules.IFrontend.MsgType_MSG_OPTIMIZATION,\
             "The grid for this launch is configured to execute only {:d} blocks, which is less than the GPU's {:d} multiprocessors."\
             " This can underutilize some multiprocessors. If you do not intend to execute this kernel concurrently with other workloads,"\
             " consider reducing the block size to have at least one block per multiprocessor or increase the size of the grid to fully utilize the available hardware resources."\
             "{}".format(int(grid_size), int(num_sms), doc_msg), "Small Grid")
-        fe.focus_metric(msg_id, "launch__grid_size", grid_size, NvRules.IFrontend.Severity_SEVERITY_HIGH, "{} < {}".format(grid_size, num_sms))
+        improvement_percent = (num_sms - grid_size) / num_sms * 100
+        fe.speedup(msg_id, NvRules.IFrontend.SpeedupType_LOCAL, improvement_percent)
+        fe.focus_metric(msg_id, "launch__grid_size", grid_size, NvRules.IFrontend.Severity_SEVERITY_HIGH, "Increase the grid size towards the number of multiprocessors ({:d})".format(num_sms))
     elif grid_size < 2 * num_sms:
-        msg_id = fe.message(NvRules.IFrontend.MsgType_MSG_WARNING,\
-        "If you execute __syncthreads() to synchronize the threads of a block, it is recommended to have more than the achieved {:d} blocks per multiprocessor."\
-        " This way, blocks that aren't waiting for __syncthreads() can keep the hardware busy.".format(int(grid_size / num_sms)), "Small Grid")
-        fe.focus_metric(msg_id, "launch__grid_size", grid_size, NvRules.IFrontend.Severity_SEVERITY_LOW, "{} < 2 * {}".format(grid_size, num_sms))
+        msg_id = fe.message(NvRules.IFrontend.MsgType_MSG_OPTIMIZATION,\
+            "If you execute __syncthreads() to synchronize the threads of a block, it is recommended to have more than the achieved {:d} blocks per multiprocessor."\
+            " This way, blocks that aren't waiting for __syncthreads() can keep the hardware busy.".format(int(grid_size / num_sms)), "Small Grid")
+        fe.focus_metric(msg_id, "launch__grid_size", grid_size, NvRules.IFrontend.Severity_SEVERITY_LOW, "Increase the grid size towards twice the number of multiprocessors ({:d})".format(2 * num_sms))
 
     partial_waves, whole_waves = math.modf(num_waves)
     partial_wave_blocks = 0. if num_waves == 0. else int(grid_size * (partial_waves / num_waves))
     potential_tail_effect = 0. if partial_waves == 0 else 1. / (whole_waves + 1.)
     if whole_waves >= 1. and potential_tail_effect >= 0.2 and achieved_occ < theoretical_occ * 0.8 and theoretical_occ > 0.:
-        fe.message(NvRules.IFrontend.MsgType_MSG_WARNING, \
+        msg_id = fe.message(NvRules.IFrontend.MsgType_MSG_OPTIMIZATION, \
             "A wave of thread blocks is defined as the maximum number of blocks that can be executed in parallel on the target GPU."\
             " The number of blocks in a wave depends on the number of multiprocessors and the theoretical occupancy of the kernel."\
             " This kernel launch results in {:d} full waves and a partial wave of {:d} thread blocks. Under the assumption of a uniform execution duration of all thread blocks,"\
@@ -89,3 +95,6 @@ def apply(handle):
             " Try launching a grid with no partial wave. The overall impact of this tail effect also lessens with the number of full waves executed for a grid."\
             "{}".format(int(whole_waves), partial_wave_blocks, 100. * potential_tail_effect, 100. * (theoretical_occ - achieved_occ) / theoretical_occ, doc_msg), \
             "Tail Effect")
+        improvement_percent = potential_tail_effect * 100
+        fe.speedup(msg_id, NvRules.IFrontend.SpeedupType_LOCAL, improvement_percent)
+        fe.focus_metric(msg_id, "launch__waves_per_multiprocessor", num_waves, NvRules.IFrontend.Severity_SEVERITY_DEFAULT, "Decrease the number of partial waves (the fractional part of the number of waves)")
