@@ -95,7 +95,7 @@ def apply(handle):
 
     if smSolPct < no_bound_threshold and memSolPct < no_bound_threshold:
         if smSolPct < latency_bound_threshold and memSolPct < latency_bound_threshold:
-            msg_type = NvRules.IFrontend.MsgType_MSG_WARNING
+            msg_type = NvRules.IFrontend.MsgType_MSG_OPTIMIZATION
             if num_waves < waves_threshold:
                 focus_metrics.append((num_waves_name, num_waves, NvRules.IFrontend.Severity_SEVERITY_HIGH, "{:.3f} < {:.3f}".format(num_waves, waves_threshold)))
                 message = "This kernel grid is too small to fill the available resources on this device, resulting in only {:.1f} full waves across all SMs. Look at @section:LaunchStats:Launch Statistics@ for more details.".format(num_waves)
@@ -106,7 +106,7 @@ def apply(handle):
                 message = "This kernel exhibits low compute throughput and memory bandwidth utilization relative to the peak performance of this device. Achieved compute throughput and/or memory bandwidth below {:.1f}% of peak typically indicate latency issues. Look at @section:SchedulerStats:Scheduler Statistics@ and @section:WarpStateStats:Warp State Statistics@ for potential reasons.".format(latency_bound_threshold)
                 name = "Latency Issue"
         elif math.fabs(smSolPct - memSolPct) >= balanced_threshold:
-            msg_type = NvRules.IFrontend.MsgType_MSG_WARNING
+            msg_type = NvRules.IFrontend.MsgType_MSG_OPTIMIZATION
             if smSolPct > memSolPct:
                 focus_metrics.append((sm_sol_pct_name, memSolPct, NvRules.IFrontend.Severity_SEVERITY_LOW, "{:.3f} - {:.3f} >= {:.3f}".format(smSolPct, memSolPct, balanced_threshold)))
                 message = "Compute is more heavily utilized than Memory: Look at the {} section to see what the compute pipelines are spending their time doing. Also, consider whether any computation is redundant and could be reduced or moved to look-up tables.".format(bottleneck_section)
@@ -131,4 +131,3 @@ def apply(handle):
     msg_id = fe.message(msg_type, message, name)
     for focus_metric in focus_metrics:
         fe.focus_metric(msg_id, focus_metric[0], focus_metric[1], focus_metric[2], focus_metric[3])
-
