@@ -50,13 +50,13 @@ class Report:
                     dash_args.append('--' + arg)
             return self.add_argument(*dash_args, **kwargs)
 
-        def parse_optional_dashless_args(self, args):
+        def parse_optional_dashless_args(self, args, parsed_args=None):
             formatted_args = []
             for arg in args:
                 if arg.split('=')[0] in self._options:
                     arg = '--' + arg
                 formatted_args.append(arg)
-            return self.parse_args(formatted_args)
+            return self.parse_args(formatted_args, namespace=parsed_args)
 
     # SQL Aggregate function that takes two arguments: start and end.  Finds the
     # total duration where at least one range is active, but doesn't over-count
@@ -184,6 +184,12 @@ class Report:
         f'pragma temp.cache_size=-{32 * 1024}', # Set temp DB page cache to 32MB
     ]
 
+    _arg_opts = [
+        [['start'], {'type': int, 'help': 'start time used for filtering'}],
+        [['end'],   {'type': int, 'help': 'end time used for filtering'}],
+        [['nvtx'],  {'type': str, 'help': 'NVTX range and domain for filtering'}],
+    ]
+
     script_name = None
     display_name = 'NO NAME GIVEN'
     usage = "{SCRIPT} -- NO USAGE INFORMATION PROVIDED"
@@ -198,9 +204,15 @@ class Report:
         self._dbcon = None
         self._dbcur = None
         self._dbfile = dbfile
-        self._args = args
+        self._args = []
+        self._parsed_args = None
         self._headers = []
 
+        if isinstance(args, argparse.Namespace):
+            self._parsed_args = args
+        else:
+            self._args = args
+
         # Check DB file
         if not os.path.exists(self._dbfile):
             raise self.Error_MissingDatabaseFile(self._dbfile)
@@ -215,7 +227,7 @@ class Report:
         qstr = urllib.parse.urlencode(dburi_query)
         urlstr = urllib.parse.urlunsplit(['file', '', os.path.abspath(self._dbfile), qstr, ''])
         try:
-            self._dbcon = sqlite3.connect(urlstr, isolation_level=None, uri=True)
+            self._dbcon = sqlite3.connect(urlstr, isolation_level=None, uri=True, check_same_thread=False)
         except sqlite3.Error:
             self._dbcon = None
             raise self.Error_InvalidDatabaseFile(self._dbfile)
@@ -230,6 +242,7 @@ class Report:
             raise self.Error_InvalidDatabaseFile(self._dbfile)
 
         self._tables = set(r[0] for r in cur.fetchall())
+        self._dbfile_basename = os.path.basename(os.path.splitext(self._dbfile)[0])
 
     def __del__(self):
         if self._dbcon != None:
@@ -267,7 +280,12 @@ class Report:
         parser = self.ArgumentParser(allow_abbrev=False)
         for opt in self._get_arg_options():
             parser.add_optional_arg(*opt[0], **opt[1])
-        self.parsed_args = parser.parse_optional_dashless_args(self.args)
+        self._parsed_args = parser.parse_optional_dashless_args(self._args, self._parsed_args)
+
+        errmsg = self.filter_time_range(self._parsed_args.start,
+            self._parsed_args.end, self._parsed_args.nvtx)
+        if errmsg != None:
+            return errmsg
 
     def get_statements(self):
         return self.statements
@@ -346,7 +364,11 @@ class Report:
             if row:
                 return (None, row[0], row[1], row[2])
 
-        return ("NVTX range '{NVTX_RANGE}' could not be found.".format(NVTX_RANGE = nvtx), None, None, None)
+        errmsg = "NVTX range '{NVTX_RANGE}' could not be found in {DBFILE}.".format(
+            NVTX_RANGE = nvtx,
+            DBFILE = self.dbfile)
+
+        return (errmsg, None, None, None)
 
     # Filters tables according to start, end, and nvtx flags, if applicable.
     # Tables that should NOT be filtered (e.g. those used for correlation ID matching)
@@ -405,13 +427,21 @@ class Report:
         return self._dbfile
 
     @property
-    def args(self):
-        return self._args
+    def dbfile_basename(self):
+        return self._dbfile_basename
+
+    @property
+    def parsed_args(self):
+        return self._parsed_args
 
     @property
     def headers(self):
         return self._headers
 
+    @property
+    def dbcon(self):
+        return self._dbcon
+
     @classmethod
     def get_script_name(klass):
         if klass.script_name == None:
@@ -432,6 +462,7 @@ class Report:
     def get_usage(klass):
         return klass.usage.format(
             SCRIPT=klass.get_script_name(),
+            DISPLAY_NAME=klass.get_display_name(),
             ROW_LIMIT=klass.DEFAULT_ROW_LIMIT)
 
     @classmethod
@@ -449,7 +480,7 @@ class Report:
         return opts
 
     @classmethod
-    def Report(klass, dbfile, args):
+    def Setup(klass, dbfile, args):
         try:
             report = klass(dbfile, args)
         except (klass.Error_MissingDatabaseFile, klass.Error_InvalidDatabaseFile) as err:
@@ -470,6 +501,14 @@ class Report:
         if errmsg != None:
             return None, klass.EXIT_SCRIPT, errmsg
 
+        return report, None, None
+
+    @classmethod
+    def Report(klass, dbfile, args):
+        report, exitval, errmsg = klass.Setup(dbfile, args)
+        if report == None:
+            return None, exitval, errmsg
+
         errmsg = report.start_query()
         if errmsg != None:
             return None, klass.EXIT_SCRIPT, errmsg
@@ -513,9 +552,6 @@ class ExpertSystemsReport(Report):
 
     _arg_opts = [
         [['rows'],  {'type': int, 'help': 'max rows', 'default': DEFAULT_ROW_LIMIT}],
-        [['start'], {'type': int, 'help': 'start time used for filtering'}],
-        [['end'],   {'type': int, 'help': 'end time used for filtering'}],
-        [['nvtx'],  {'type': str, 'help': 'NVTX range and domain for filtering'}],
     ]
 
     def setup(self):
@@ -525,11 +561,6 @@ class ExpertSystemsReport(Report):
 
         self._row_limit = self.parsed_args.rows
 
-        err = self.filter_time_range(self.parsed_args.start,
-            self.parsed_args.end, self.parsed_args.nvtx)
-        if err != None:
-            return err
-
     message_advice = "NO ADVICE MESSAGE DEFINED"
     message_noresult = "NO NON-RESULT MESSAGE DEFINED"
 
