@@ -4,25 +4,9 @@ import nsysstats
 
 class CUDAAPITrace(nsysstats.StatsReport):
 
-    display_name = 'CUDA API Trace'
-    usage = f"""{{SCRIPT}} -- CUDA API Trace
-
-    No arguments.
-
-    Output: All time values default to nanoseconds
-        Start : Timestamp when API call was made
-        Duration : Length of API calls
-        Name : API function name
-        Result : Return value of API call
-        CorrID : Correlation used to map to other CUDA calls
-        Pid : Process ID that made the call
-        Tid : Tread ID that made the call
-        T-Pri : Run priority of call thread
-        Thread Name : Name of thread that called API function
-
-    This report provides a trace record of CUDA API function calls and
-    their execution times.
-"""
+    display_name = 'DEPRECATED - Use cuda_api_trace instead'
+    usage = '{SCRIPT} -- {{DISPLAY_NAME}}'
+    should_display = False
 
     query = """
 SELECT
